/*
Eteria IRC Client, an RFC 1459 compliant client program written in Java.
Copyright (C) 2000  Javier Kohen <jkohen at tough.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class ChannelList extends Frame {
    Table table;
    ServerProcess sp;

    ChannelList(ServerProcess sp_param, Vector list) {
	super();

	this.sp = sp_param;

	setTitle(sp.getString("channel_list.s1"));

	GridBagLayout gb = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();

	setLayout(gb);

	gbc.insets = new Insets(4, 4, 4, 4);

	Button b;
	Label l;

	int max_width = Toolkit.getDefaultToolkit().getScreenSize().width / 10 * 8;

	table = new Table(0, 3, 8);
	table.setMaxWidth(max_width);
	table.addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent e) {
		    if (2 != e.getClickCount()) {
			return;
		    }
		    Point cell = table.getCellFromCoord(e.getPoint());
		    if (-1 == cell.x || -1 == cell.y) {
			return;
		    }
		    Vector v = table.getRow(cell.y);
		    joinChannel(new Channel((String) v.elementAt(0), (String) v.elementAt(1), (String) v.elementAt(2)));
		}
	    });
	{
	    String p[] = {
		sp.getString("channel_list.s2"),
		sp.getString("channel_list.s3"),
		sp.getString("channel_list.s4")
	    };
	    table.setHeader(p);
	}
	table.selectWholeRow(true);
	loadList(list);
	gbc.fill = GridBagConstraints.BOTH;
	gbc.gridwidth = GridBagConstraints.REMAINDER;
  	gbc.weightx = gbc.weighty = 1.0;
	gb.setConstraints(table, gbc);
	add(table);
	gbc.fill = GridBagConstraints.NONE;
	gbc.gridwidth = 1;
	gbc.weightx = gbc.weighty = 0.0;

	gbc.gridy = 1;

	gbc.weightx = 1.0;

	b = new Button(sp.getString("channel_list.s5"));
	b.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ev) {
		    String[] s = (String[]) table.getSelectedObjects();
		    if (null != s) {
			joinChannel(new Channel(s[0], s[1], s[2]));
		    }
		}
	    });
	gb.setConstraints(b, gbc);
	add(b);

	b = new Button(sp.getString("channel_list.s6"));
	b.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ev) {
		    String p[] = { sp.getConfString("list_parameter") };
		    sp.sendMessage("list", p);
		}
	    });
	gb.setConstraints(b, gbc);
	add(b);

	b = new Button(sp.getString("cancel"));
	b.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ev) {
		    dispose();
		}
	    });
	gb.setConstraints(b, gbc);
	add(b);

	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    dispose();
		}
	    });
    }

    public void loadList(Vector list) {
	table.removeAll();

	for (Enumeration e = list.elements(); e.hasMoreElements(); ) {
	    Channel chan = (Channel) e.nextElement();

	    String name = chan.getName();
	    String users = chan.getUsers();
	    String topic = chan.getTopic();
	    int prio = chan.getPriority();

	    if (-1 != prio) {
		table.add(prio * 3, name);
		table.add(prio * 3 + 1, users);
		table.add(prio * 3 + 2, topic);
	    } else {
		table.add(name);
		table.add(users);
		table.add(topic);
	    } 
	}
    }

    void joinChannel(Channel chan) {
	String name = chan.getName();
	ChannelWindow cw = sp.getChannel(name);
	if (null == cw) {
	    String p[] = { name };
	    sp.sendMessage("join", p);
	} else {
	    sp.showPanel(name);
	}
    }

    int getPreferredWidth() {
	int screen_width = Toolkit.getDefaultToolkit().getScreenSize().width;
	int normal_width = super.getPreferredSize().width;

	return Math.min(normal_width, screen_width);
    }

    public Dimension getPreferredSize() {
	Dimension normal_size = super.getPreferredSize();

	return new Dimension(getPreferredWidth(), normal_size.height);
    }

    public void setBackground(Color c) {
	super.setBackground(c);
	table.setBackground(c);
    }

    public void setForeground(Color c) {
	super.setForeground(c);
	table.setForeground(c);
    }
}
