/*
Eteria IRC Client, an RFC 1459 compliant client program written in Java.
Copyright (C) 2000  Javier Kohen <jkohen at tough.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import java.text.*;
import java.util.Date;

import com.lyrisoft.awt.*;
import htmllayout.HtmlLayout;

class ChannelWindow extends ChatWindow implements History, ActionListener, MouseListener {
    private TextField topic;
    private TextFieldHistory entry;
    private NickList nicks;
    private ImageButton whois;
    private ImageButton close;
    private PopupMenu popup_menu;

    // Purge entries past this number.
    private int history_max_size = 200;
    private int rows = 0;

    private ServerProcess sp;
    private String channel;
    private boolean is_active = false;

    private NickInfo item_under_menu;

    public ChannelWindow(ServerProcess spr, String chan, String title) {
	super(title);

  	this.sp = spr;
	this.channel = chan;

//  	String html_layout;

//  	try {
//  	    html_layout = new String(Resource.getResource("ChannelWindow.html", this));
//  	} catch (java.io.IOException e) {
//  	    System.err.println(e);
//  	    return;
//  	}

//  	setLayout(new HtmlLayout(html_layout));

	GridBagLayout gb = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();

	setLayout(gb);

	gbc.insets = new Insets(2, 2, 2, 2);
	gbc.gridy = 0;

	/* Component's initialization
	 */

	topic = new TextField();
	gbc.weightx = 1.0;
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gb.setConstraints(topic, gbc);
	add(topic/*, "topic"*/);
	gbc.fill = GridBagConstraints.NONE;
	gbc.weightx = 0.0;
	gbc.gridwidth = 1;

	/* Next Row */
	gbc.gridy++;

	gbc.weightx = 1.0;
	gbc.weighty = 1.0;
	gbc.fill = GridBagConstraints.BOTH;
	gbc.gridheight = GridBagConstraints.RELATIVE;
	gb.setConstraints(scroll_text_canvas, gbc);
	add(scroll_text_canvas/*, "scroll_text_canvas"*/);
	gbc.gridheight = 1;
	gbc.fill = GridBagConstraints.NONE;
	gbc.weighty = 0.0;
	gbc.weightx = 0.0;

	Panel right_panel = new Panel();
	GridBagLayout right_gb = new GridBagLayout();
	right_panel.setLayout(right_gb);

	/* Backup Grid's Y-Pos. */
	int old_gridy = gbc.gridy;

	/* Right Panel begins here. */
	gbc.weightx = 1.0;

	String nl_titles[] = new String[NickList.MODES];
	for (int i = 0; i < NickList.MODES; i++) {
	    nl_titles[i] = sp.getString("nicklist.s1." + i);
	}
	nicks = new NickList(nl_titles);
	gbc.weighty = 1.0;
	gbc.fill = GridBagConstraints.BOTH;
	right_gb.setConstraints(nicks, gbc);
	right_panel.add(nicks/*, "nicks"*/);
	gbc.fill = GridBagConstraints.NONE;
	gbc.weighty = 0.0;

	/* Next Row */
	gbc.gridy++;

	whois = new ImageButton();
	whois.setImage(ImageButton.ENTERED,
		       sp.getImage("channel.whois.entered"));
	whois.setImage(ImageButton.EXITED,
		       sp.getImage("channel.whois.exited"));
	whois.setImage(ImageButton.PRESSED,
		       sp.getImage("channel.whois.pressed"));
	gbc.anchor = GridBagConstraints.WEST;
	right_gb.setConstraints(whois, gbc);
	right_panel.add(whois/*, "whois"*/);

	/* Next Row */
	gbc.gridy++;

	close = new ImageButton();
	close.setImage(ImageButton.ENTERED,
		       sp.getImage("channel.close.entered"));
	close.setImage(ImageButton.EXITED,
		       sp.getImage("channel.close.exited"));
	close.setImage(ImageButton.PRESSED,
		       sp.getImage("channel.close.pressed"));
	right_gb.setConstraints(close, gbc);
	right_panel.add(close/*, "close"*/);
	gbc.anchor = GridBagConstraints.CENTER;

	gbc.weightx = 0.0;

	/* Right Panel ends here. */

	/* Restore Grid's Y-Pos. */
	gbc.gridy = old_gridy;

	gbc.weighty = 1.0;
	gbc.fill = GridBagConstraints.BOTH;
	gbc.gridheight = GridBagConstraints.RELATIVE;
	gb.setConstraints(right_panel, gbc);
	add(right_panel);
	gbc.gridheight = 1;
	gbc.fill = GridBagConstraints.NONE;
	gbc.weighty = 0.0;

	/* Next Row */
	gbc.gridy++;

	entry = new TextFieldHistory(80);
	gbc.weightx = 1.0;
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gb.setConstraints(entry, gbc);
	add(entry/*, "entry"*/);
	gbc.fill = GridBagConstraints.NONE;
	gbc.gridwidth = 1;
	gbc.weightx = 0.0;

	popup_menu = new PopupMenu();
	// We delay the building of the menu up to the moment when our nick is added to the list. This is needed in order to know whether we are opped or not.
	add(popup_menu);

	/* Event Listeners
	 */

	topic.addActionListener(this);
	popup_menu.addActionListener(this);
	nicks.addActionListener(this);
	nicks.addMouseListener(this);
	entry.addActionListener(this);
  	whois.addMouseListener(this);
  	close.addMouseListener(this);
    }

    public void requestFocus() {
	entry.requestFocus();
    }

    public void trimHistory(int new_size) {
//  	int end;

//  	while (new_size < rows) {
//  	    String t = text_canvas.getText();

//  	    if (-1 == (end = t.indexOf('\n'))) {
//  		break;
//  	    }

//  	    // Replace extra lines with nothing.
//  	    text_canvas.replaceRange("", 0, end + 1);
//  	    rows--;
//  	}
    }

    public void output(String s) {
	int i = 0;
	// Count rows in input.
	while (-1 != (i = s.indexOf('\n', i + 1))) {
	    rows++;
	}
	trimHistory(history_max_size);
 	text_canvas.append(s);
    }

    protected String getNick() {
	return sp.getNick();
    }

    protected boolean getConfBoolean(String key) {
	return sp.getConfBoolean(key);
    }

    protected String getString(String key) {
	return sp.getString(key);
    }

    protected void visitURL(URL url) {
	sp.visitURL(url);
    }

    protected void joinChannel(String name) {
	ChannelWindow cw = sp.getChannel(name);
	if (null == cw) {
	    String p[] = { name };
	    sp.sendMessage("join", p);
	} else {
	    sp.showPanel(name);
	}
    }

    void addNick(String n) {
	nicks.add(n);

	if (NickList.canonicalize(n).equals(sp.getNick())) {
	    buildPopupMenu(popup_menu);
	}
    }

    void addNicks(String [] n) {
	nicks.setVisible(false);

	for (int i = 0; i < n.length; i++) {
	    addNick(n[i]);
	}

	nicks.setVisible(true);

  	validate();
    }

    void addNick(String n, int mode) {
	nicks.add(n, mode);

	if (n.equals(sp.getNick())) {
	    buildPopupMenu(popup_menu);
	}
    }

    void removeNick(String n) {
	nicks.remove(n);
    }

    void changeNick(String n_1, String n) {
	nicks.change(n_1, n);
    }

    void changeMode(String n, char mode, boolean state) {
	int m = NickList.modeCharToMode(mode);

	if (state) {
	    nicks.add(n, m);
	} else {
	    nicks.remove(n, m);
	}

	if (n.equals(sp.getNick())) {
	    buildPopupMenu(popup_menu);
	}
    }

    void setTopic(String s) {
	topic.setText(s);
    }

    public int getMaximumHistorySize() {
	return history_max_size;
    }

    public void setMaximumHistorySize(int new_size) {
	history_max_size = new_size; 
	trimHistory(history_max_size);
    }

    public void setTextBackground(Color c) {
	text_canvas.setBackground(c);
	entry.setBackground(c);
	topic.setBackground(c);
    }

    public void setTextForeground(Color c) {
	text_canvas.setForeground(c);
	entry.setForeground(c);
	topic.setForeground(c);
    }

    public void setBackground(Color c) {
	super.setBackground(c);
  	nicks.setBackground(c);
    }

    public void setForeground(Color c) {
	super.setForeground(c);
  	nicks.setForeground(c);
    }

    synchronized void buildPopupMenu(PopupMenu pm) {
	Menu mn;
	MenuItem mi;

	pm.removeAll();

	mi = new MenuItem(sp.getString("nicklist.popup.query"));
	mi.setActionCommand("query");
	pm.add(mi);
	pm.addSeparator();
	mi = new MenuItem(sp.getString("nicklist.popup.ping"));
	mi.setActionCommand("ping");
	pm.add(mi);
	mi = new MenuItem(sp.getString("nicklist.popup.whois"));
	mi.setActionCommand("whois");
	pm.add(mi);
	pm.addSeparator();
	mi = new MenuItem(sp.getString("nicklist.popup.ignore"));
	mi.setActionCommand("ignore");
	pm.add(mi);
	mi = new MenuItem(sp.getString("nicklist.popup.unignore"));
	mi.setActionCommand("unignore");
	pm.add(mi);

	if (nicks.isOp(sp.getNick())) {
	    pm.addSeparator();
	    mn = new Menu(sp.getString("nicklist.popup.operators"));
	    mn.addActionListener(this);
	    mi = new MenuItem(sp.getString("nicklist.popup.kick"));
	    mi.setActionCommand("kick " + channel);
	    mn.add(mi);
	    mi = new MenuItem(sp.getString("nicklist.popup.kban"));
	    mi.setActionCommand("kban");
	    mn.add(mi);
	    mn.addSeparator();
	    mi = new MenuItem(sp.getString("nicklist.popup.ban"));
	    mi.setActionCommand("mode " + channel + " +b");
	    mn.add(mi);
	    mi = new MenuItem(sp.getString("nicklist.popup.unban"));
	    mi.setActionCommand("mode " + channel + " -b");
	    mn.add(mi);
	    mn.addSeparator();
	    mi = new MenuItem(sp.getString("nicklist.popup.voice"));
	    mi.setActionCommand("mode " + channel + " +v");
	    mn.add(mi);
	    mi = new MenuItem(sp.getString("nicklist.popup.unvoice"));
	    mi.setActionCommand("mode " + channel + " -v");
	    mn.add(mi);
	    mn.addSeparator();
	    mi = new MenuItem(sp.getString("nicklist.popup.op"));
	    mi.setActionCommand("mode " + channel + " +o");
	    mn.add(mi);
	    mi = new MenuItem(sp.getString("nicklist.popup.deop"));
	    mi.setActionCommand("mode " + channel + " -o");
	    mn.add(mi);
	    if (sp.getConfBoolean("have_halfops")) {
		mn.addSeparator();
		mi = new MenuItem(sp.getString("nicklist.popup.hop"));
		mi.setActionCommand("mode " + channel + " +h");
		mn.add(mi);
		mi = new MenuItem(sp.getString("nicklist.popup.dehop"));
		mi.setActionCommand("mode " + channel + " -h");
		mn.add(mi);
	    }
	    pm.add(mn);
	}
    }

    public void actionPerformed(ActionEvent ev) {
	Object comp = ev.getSource();

	if (comp instanceof MenuItem) {
	    sp.sendCommand(ev.getActionCommand() + " " + item_under_menu.getNick(), this);
	} else if (comp.equals(topic)) {
	    String p[] = { channel, topic.getText() };
	    sp.sendMessage("topic", p);
	} else if (comp.equals(nicks)) {
	    String item = ev.getActionCommand();
	    if (!NickList.isTitle(item)) {
		sp.openPrivate(item);
		sp.showPanel(item);
	    }
	} else if (comp.equals(entry)) {
	    boolean isCommand = false;

	    String text = entry.getText();
	    if (0 == text.length()) {
		return;
	    }

	    if (text.charAt(0) == '/') {
		if (1 == text.length()) {
		    return;
		}
		isCommand = true;
		text = text.substring(1);
	    }

	    if (isCommand) {
		Component source = (Component) ev.getSource();
		sp.sendCommand(text, (OutputWindow) source.getParent());
	    } else {
		String p[] = { channel, entry.getText() };
		sp.sendMessage("privmsg", p);
		printMyPrivmsg(entry.getText());
	    }

	    entry.setText("");
	}
    }

    public void mouseClicked(MouseEvent ev) {
	Component comp = ev.getComponent();

	if (comp.equals(whois)) {
	    boolean selected = false;
	    NickInfo[] selection = (NickInfo[]) nicks.getSelectedObjects();
	    if (null != selection) {
		String s = "";
		for (int i = 0; i < selection.length; i++) {
		    if (!selection[i].isTitle()) {
			if (selected) {
			    s = s.concat(",");
			}
			selected = true;
			s = s.concat(selection[i].getNick());
		    }
		}

		// Was a nick selected?
		if (selected) {
		    String p[] = { s };
		    sp.sendMessage("whois", p);
		}
	    }

	    if (!selected) {
		printError(sp.getString("eirc.select_nicks"));
	    }
	} else if (comp.equals(close)) {
	    dispose();
	}
    }

    public void mousePressed(MouseEvent ev) {
        Object source = ev.getSource();

	if (source.equals(nicks)) {
	    if (ev.isPopupTrigger() ||
		0 != (ev.getModifiers() & MouseEvent.BUTTON3_MASK)) { // WORKAROUND: #27 MSIE doesn't set popupTrigger.
		NickInfo item = nicks.getNickInfoAt(ev.getPoint());
		if (null != item && !item.isTitle()) {
		    item_under_menu = item;
		    popup_menu.show(nicks, ev.getX() - 5, ev.getY() - 5);
		}
	    }
	}
    }

    public void mouseReleased(MouseEvent ev) {
    }

    public void mouseEntered(MouseEvent ev) {
    }

    public void mouseExited(MouseEvent ev) {
    }
}
