/*
Eteria IRC Client, an RFC 1459 compliant client program written in Java.
Copyright (C) 2000  Javier Kohen <jkohen at tough.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.*;
import java.awt.event.*;

public class ChatPanel extends Panel {
    protected boolean is_disposed;
    protected String panelName;

    protected ChatPanelListener chatPanelListener;
    protected TextListener textListener;

    public ChatPanel(String name) {
	this.panelName = name;
	this.is_disposed = false;
    }

    public String getPanelName() {
	return panelName;
    }

    public void setPanelName(String panelName) {
	this.panelName = panelName;
    }

    synchronized public void dispose() {
	if (is_disposed) {
	    return;
	}

	postChatPanelClosingEvent();
	is_disposed = true;
    }

    public void addChatPanelListener(ChatPanelListener l) {
	if (null != l) {
	    chatPanelListener = MyAWTEventMulticaster.add(chatPanelListener, l);
	}
    }

    public void removeChatPanelListener(ChatPanelListener l) {
	if (null != l) {
	    chatPanelListener = MyAWTEventMulticaster.remove(chatPanelListener, l);
	}
    }

    synchronized void postChatPanelClosingEvent() {
	if (chatPanelListener != null) {
	    chatPanelListener.chatPanelClosing(new ChatPanelEvent(this, ChatPanelEvent.CHATPANEL_CLOSING));
	}
    }

    public void addTextListener(TextListener l) {
	if (null != l) {
	    textListener = AWTEventMulticaster.add(textListener, l);
	}
    }

    public void removeTextListener(TextListener l) {
	if (null != l) {
	    textListener = AWTEventMulticaster.remove(textListener, l);
	}
    }

    synchronized void postTextEvent() {
	if (textListener != null) {
	    textListener.textValueChanged(new TextEvent(this, TextEvent.TEXT_VALUE_CHANGED));
	}
    }
}
