/*
Eteria IRC Client, an RFC 1459 compliant client program written in Java.
Copyright (C) 2000  Javier Kohen <jkohen at tough.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;
import java.util.Hashtable;

public class ChatPanelContainer extends Panel implements ActionListener, MouseListener, TextListener, KeyListener {
    private ServerProcess sp;

    private Hashtable windows_list;
    private OutputWindow status;
    private String status_tag;

    private String revert_to;

    private CardLayout windows_layout;
    private Panel windows;
    private ChatTabs tabs;
    private PopupMenu tabs_popup;
    private ImageCanvas label;
    private Panel bottom_panel;
    private Button shift_left;
    private Button shift_right;

    private String tab_under_menu;

    ChatPanelContainer(ServerProcess sp) {
	this.sp = sp;

	windows_list = new Hashtable();

	GridBagLayout gb = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();

	setLayout(gb);

	gbc.insets = new Insets(0, 2, 0, 2);

	windows = new Panel();
	gbc.weighty = 1.0;
	gbc.weightx = 1.0;
	gbc.fill = GridBagConstraints.BOTH;
	gb.setConstraints(windows, gbc);
	add(windows);
	gbc.weightx = 0.0;
	gbc.weighty = 0.0;

	gbc.gridy = 1;

	bottom_panel = new Panel();
	GridBagLayout bottom_gb = new GridBagLayout();
	bottom_panel.setLayout(bottom_gb);

	/* Backup Grid's Y-Pos. */
	int old_gridy = gbc.gridy;

	/* Bottom Panel begins here. */
	gbc.gridy = 0;

	label = new ImageCanvas(sp.getImage("tabs.label"));
	gbc.gridheight = GridBagConstraints.REMAINDER;
	bottom_gb.setConstraints(label, gbc);
	bottom_panel.add(label);
	gbc.gridheight = 1;

  	tabs = new ChatTabs();
	gbc.weightx = 1.0;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gbc.gridheight = GridBagConstraints.REMAINDER;
	bottom_gb.setConstraints(tabs, gbc);
	bottom_panel.add(tabs);
	gbc.gridheight = 1;
	gbc.fill = GridBagConstraints.NONE;
	gbc.weightx = 0.0;

	shift_left = new Button("<");
	bottom_gb.setConstraints(shift_left, gbc);
	bottom_panel.add(shift_left);

	gbc.gridy++;

	shift_right = new Button(">");
	bottom_gb.setConstraints(shift_right, gbc);
	bottom_panel.add(shift_right);
	
	/* Bottom Panel ends here. */

	/* Restore Grid's Y-Pos. */
	gbc.gridy = old_gridy;

	gbc.weightx = 1.0;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gb.setConstraints(bottom_panel, gbc);
	add(bottom_panel);

	tabs.setImage(ChatTabs.NORMAL, sp.getImage("tabs.normal"));
	tabs.setImage(ChatTabs.CURRENT, sp.getImage("tabs.current"));
	tabs.setImage(ChatTabs.ALARM, sp.getImage("tabs.alarm"));

	windows_layout = new CardLayout();
	windows.setLayout(windows_layout);

	MenuItem mi;
	tabs_popup = new PopupMenu();
	mi = new MenuItem(sp.getString("open"));
	mi.setActionCommand("open");
	tabs_popup.add(mi);
	mi = new MenuItem(sp.getString("close"));
	mi.setActionCommand("close");
	tabs_popup.add(mi);
	add(tabs_popup);

//  	tabs.addKeyListener(this);
	tabs.addActionListener(this);
	tabs.addMouseListener(this);
	shift_left.addActionListener(this);
	shift_right.addActionListener(this);
	tabs_popup.addActionListener(this);
    }

    public void showStatus() {
	show(status_tag);
    }

    public void show(String name) {
	ChatPanel cp = (ChatPanel) windows_list.get(name);
	if (null == cp) {
	    cp = status;
	}

	windows_layout.show(windows, name);
	cp.requestFocus();
	tabs.setAlarm(name, false);
  	tabs.makeVisible(name);
    }

    public Enumeration chats() {
	return windows_list.keys();
    }

    public OutputWindow getStatus() {
	return status;
    }

    public void addStatus(OutputWindow w, String tag) {
	this.status = w;
	this.status_tag = tag;

	windows.add(w, status_tag);
	tabs.add(status_tag);
  	show(status_tag);

//  	w.addKeyListener(this);
	w.addTextListener(this);
    }

    public void removeStatus() {
	tabs.remove(status_tag);
	windows.remove(status);
	this.status = null;
    }

    public ChatWindow getChat(String name) {
	return (ChatWindow) windows_list.get(name);
    }

    public void addChat(ChatWindow w, String name) {
	windows_list.put(name, w);
	windows.add(w, name);
	tabs.add(name);

//  	w.addKeyListener(this);
	w.addTextListener(this);
    }

    public void removeChat(String name) {
	tabs.remove(name);
	Component c = (Component) windows_list.remove(name);
	windows.remove(c);

	show(getRevertTo());
    }

    public void renameChat(String old_name, String name) {
	ChatWindow w = (ChatWindow) windows_list.get(old_name);

	// Removing the old window isn't necessary. We just add it with another name.
	windows.add(w, name);
	if (old_name.equals(tabs.getVisible())) {
	    windows_layout.show(windows, name);
	}

	// FIXME: Must use an atomic operator.
	w.setPanelName(name);
	windows_list.put(name, w);
  	windows_list.remove(old_name);

	tabs.rename(old_name, name);
    }

    public String getRevertTo() {
	return (windows_list.containsKey(revert_to) ? revert_to : status_tag);
    }

    public void setRevertTo(String target) {
	this.revert_to = target;
    }

    public void setForeground(Color c) {
	super.setForeground(c);
	windows.setForeground(c);
	bottom_panel.setForeground(c);
  	label.setForeground(c);
  	tabs.setForeground(c);
    }

    public void setBackground(Color c) {
	super.setBackground(c);
	windows.setBackground(c);
	bottom_panel.setBackground(c);
  	label.setBackground(c);
  	tabs.setBackground(c);
    }

    public void actionPerformed(ActionEvent ev) {
	Object source = ev.getSource();

	if (source.equals(tabs)) {
	    show(ev.getActionCommand());
	} else if (source.equals(shift_left)) {
	    tabs.shiftLeft();
	} else if (source.equals(shift_right)) {
	    tabs.shiftRight();
	} else if (source instanceof MenuItem) {
	    String cmd = ev.getActionCommand();

	    if (cmd.equals("open")) {
		show(tab_under_menu);
	    } else if (cmd.equals("close")) {
		ChatWindow w = ((ChatWindow) windows_list.get(tab_under_menu));
		if (null != w) {
		    w.dispose();
		}
	    }
	}
    }

    public void mouseClicked(MouseEvent ev) {
    }

    public void mousePressed(MouseEvent ev) {
        Object source = ev.getSource();

	if (source.equals(tabs)) {
	    if (ev.isPopupTrigger() ||
		0 != (ev.getModifiers() & MouseEvent.BUTTON3_MASK)) { // WORKAROUND: #27 MSIE doesn't set popupTrigger.
		String tab = tabs.getTabAt(ev.getPoint());
  		if (null != tab) {
		    ev.consume();
  		    tab_under_menu = tab;
		    tabs_popup.show(tabs, ev.getX() - 5, ev.getY() - 5);
  		}
	    }
	}
    }

    public void mouseReleased(MouseEvent ev) {
    }

    public void mouseEntered(MouseEvent ev) {
    }

    public void mouseExited(MouseEvent ev) {
    }

    public void textValueChanged(TextEvent e) {
	ChatPanel cp = (ChatPanel) e.getSource();
	String name = cp.getPanelName();

	// Don't try to set an alarm on a recently closed window.
	if (windows_list.containsKey(name) || name.equals(status_tag)) {
	    tabs.setAlarm(name, true);
	}
    }

    public void keyTyped(KeyEvent e) {
	System.err.println("DEBUG: keyTyped = " + e);
    }

    public void keyPressed(KeyEvent e) {
	System.err.println("DEBUG: keyPressed = " + e);
    }

    public void keyReleased(KeyEvent e) {
	System.err.println("DEBUG: keyReleased = " + e);
    }
}
