/*
Eteria IRC Client, an RFC 1459 compliant client program written in Java.
Copyright (C) 2000  Javier Kohen <jkohen at tough.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.*;
import com.lyrisoft.awt.*;

public abstract class ChatWindow extends OutputWindow {
    public TextStyle MY_NICK_STYLE;
    public TextStyle ACTION_STYLE;
    public TextStyle PRIVMSG_STYLE;

    public ChatWindow(String title) {
	super(title);

	Font bold_font = new Font(DEFAULT_FONT.getName(), Font.BOLD, DEFAULT_FONT.getSize());

	MY_NICK_STYLE = new TextStyle(bold_font, Color.red);
	ACTION_STYLE = new TextStyle(DEFAULT_FONT, Color.magenta);
	PRIVMSG_STYLE = new TextStyle(DEFAULT_FONT, Color.black);
    }

    public void printMyAction(String s) {
	printAction(s, getNick());
    }

    public void printAction(String s, String whom) {
	String prefix = getString("eirc.pre.action").concat(whom).concat(" ");

	text_canvas.append(prefix.concat(s).concat("\n"), ACTION_STYLE);

	postTextEvent();
    }

    public void printMyPrivmsg(String s) {
	String prefix = "";

	if (getConfBoolean("prefix_own")) {
	    prefix = prefix.concat("<").concat(getNick());
	}
	prefix = prefix.concat("> ");

	text_canvas.append(prefix, MY_NICK_STYLE);
	text_canvas.append(s.concat("\n"), PRIVMSG_STYLE);

	postTextEvent();
    }

    public void printPrivmsg(String s, String whom) {
	String prefix = "<".concat(whom).concat("> ");

	text_canvas.append(prefix, NICK_STYLE);
	text_canvas.append(s.concat("\n"), PRIVMSG_STYLE);

	postTextEvent();
    }

    public void printMyNotice(String s) {
	String prefix = "";

	if (getConfBoolean("prefix_own")) {
	    prefix = prefix.concat("-").concat(getNick());
	}
	prefix = prefix.concat("- ");

	text_canvas.append(prefix, MY_NICK_STYLE);
	text_canvas.append(s.concat("\n"), NOTICE_STYLE);

	postTextEvent();
    }

    public void setFont(Font f) {
	super.setFont(f);

	Font bold_font = new Font(f.getName(), Font.BOLD, f.getSize());

	MY_NICK_STYLE.setFont(bold_font);
	ACTION_STYLE.setFont(f);
	PRIVMSG_STYLE.setFont(f);
    }

    /* Returns the user's current nick */
    protected abstract String getNick();
}
