/*
Eteria IRC Client, an RFC 1459 compliant client program written in Java.
Copyright (C) 2000  Javier Kohen <jkohen at tough.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.io.*;
import java.net.*;
import java.util.Enumeration;
import java.util.Vector;

public class ClientThread extends Thread implements ClientProcess {
    private Socket s;
    private ServerProcess sp;
    private BufferedReader br;
    private BufferedWriter bw;
    private boolean disconnected = false;
    
    public ClientThread(ServerProcess sp, Socket s) throws IOException {
	this.s = s;
	this.sp = sp;
	br = new BufferedReader(new InputStreamReader(s.getInputStream()));
	bw = new BufferedWriter(new OutputStreamWriter(s.getOutputStream()));
    }
    
    public void run() {
	try {
	    while (!disconnected) {
		String str = br.readLine();
		if (null == str) {
		    disconnect();
		} else if (0 != str.trim().length()) {
  		    try {
    			sp.processMessage(new RemoteMessage(str, this));
  		    } catch (java.text.ParseException e) {
  			System.err.println(e);
  		    }
		}
	    }
	} catch (SocketException e) {
	    disconnect();
	    System.err.println(e);
	} catch (IOException e) {
	    disconnect();
	    System.err.println(e);
	}

	sp.disconnect();
    }

    public void enqueueMessage(LocalMessage sm) throws IOException {
	Vector v = sm.toStrings();
	for (Enumeration e = v.elements(); e.hasMoreElements(); ) {
	    String t = (String) e.nextElement();
	    bw.write(t, 0, t.length());
	}
	bw.flush();
    }

    synchronized public void disconnect() {
	if (!disconnected) {
	    disconnected = true;

	    try {
		br.close();
	    } catch (IOException e) {
		System.err.println(e);
	    }

	    try {
		bw.close();
	    } catch (IOException e) {
		System.err.println(e);
	    }
	}
    }
}
