/*
Eteria IRC Client, an RFC 1459 compliant client program written in Java.
Copyright (C) 2000  Javier Kohen <jkohen at tough.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.MissingResourceException;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;

/*
 * This class gathers the benefits of List and Property bundles. It also permits content update, unlike the others.
 */
public class ConfigurationBundle extends ResourceBundle {
    Hashtable contents;
    String name;

    ConfigurationBundle(String[][] default_values, String configuration_source) {
	ResourceBundle configuration = ResourceBundle.getBundle(configuration_source);
	name = configuration_source;
	contents = new Hashtable(default_values.length);

	for (int i = 0; i < default_values.length; i++) {
	    String val = default_values[i][1];

	    try {
		val = configuration.getString(default_values[i][0]);
	    } catch (MissingResourceException e) {
		// Use defaults for options not in the configuration file.
	    }

	    contents.put(default_values[i][0], val);
	}
    }

    public boolean getBoolean(String key) {
	String val = getString(key);

	if (val.equals("0") ||
	    val.equals("false") ||
	    val.equals("no") ||
	    val.equals("off")) {
	    return false;
	} else if (val.equals("1") ||
		   val.equals("true") ||
		   val.equals("yes") ||
		   val.equals("on")) {
	    return true;
	}

	throw new MissingResourceException("value isn't boolean", name, key);
    }

    public int getInt(String key) {
	String val = getString(key);

	try {
	    return Integer.parseInt(val);
	} catch (NumberFormatException e) {
	    throw new MissingResourceException("value isn't int", name, key);
	}
    }

    public Object handleGetObject(String key) throws MissingResourceException {
	Object val = contents.get(key);

	if (null == val) {
	    throw new MissingResourceException("key doesn't exist", name, key);
	}

	return val;
    }

    public Enumeration getKeys() {
	return contents.keys();
    }

    public void setBoolean(String key, boolean val) {
	setString(key, val ? "1" : "0");
    }

    public void setInt(String key, int val) {
	setString(key, String.valueOf(val));
    }

    public void setString(String key, String val) {
	if (!contents.containsKey(key)) {
	    throw new MissingResourceException("key doesn't exist", name, key);
	}

	contents.put(key, val);
    }
}
