/*
Eteria IRC Client, an RFC 1459 compliant client program written in Java.
Copyright (C) 2000  Javier Kohen <jkohen at tough.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.*;
import java.awt.event.*;

class Configurator extends Frame {
    ServerProcess sp;

    List tabs;
    Panel page;
    CardLayout page_layout;

    String[][] default_options;
    Component[] components;

    Configurator(ServerProcess sp, String[][] def_opt) {
	super();

	this.sp = sp;
	default_options = def_opt;

	components = new Component[EIRC.CONF_OPTIONS_COUNT];

	setTitle(sp.getString("conf.title"));

	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    dispose();
		}
	    });

	tabs = new List(1);
	tabs.addItemListener(new ItemListener() {
		public void itemStateChanged(ItemEvent e) {
		    page_layout.show(page, tabs.getSelectedItem());
		}
	    });

	page = new Panel();

	Panel action = new Panel();
	Button b;

	b = new Button(sp.getString("ok"));
	b.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ev) {
		    updateConfiguration();
		    dispose();
		}
	    });
	action.add(b, BorderLayout.WEST);

	b = new Button(sp.getString("cancel"));
	b.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ev) {
		    dispose();
		}
	    });
	action.add(b, BorderLayout.EAST);

	add(tabs, BorderLayout.WEST);
	add(page, BorderLayout.EAST);
	add(action, BorderLayout.SOUTH);

	page_layout = new CardLayout();
	page.setLayout(page_layout);

	Panel p, p2;
	Component c;
	GridBagLayout gb;
	GridBagConstraints gbc;

	/* Miscellaneous settings Tab. */
	p = new Panel();
	p.setLayout(new GridLayout(0, 2, 2, 2));

	c = new Checkbox(sp.getString("conf.prefix_own"), sp.getConfBoolean("prefix_own"));
	p.add(c);
	components[EIRC.CONF_PREFIX_OWN] = c;

	c = new Checkbox(sp.getString("conf.see_everything_from_server"), sp.getConfBoolean("see_everything_from_server"));
	p.add(c);
	components[EIRC.CONF_SEE_EVERYTHING_FROM_SERVER] = c;

	c = new Checkbox(sp.getString("conf.see_join"), sp.getConfBoolean("see_join"));
	p.add(c);
	components[EIRC.CONF_SEE_JOIN] = c;

	c = new Checkbox(sp.getString("conf.filter_mirc_attribs"), sp.getConfBoolean("filter_mirc_attribs"));
	p.add(c);
	components[EIRC.CONF_FILTER_MIRC_ATTRIBS] = c;

	/* Quit message option. */
	p2 = new Panel();
	gb = new GridBagLayout();
	gbc = new GridBagConstraints();
	p2.setLayout(gb);

	c = new Label(sp.getString("conf.quit_message"), Label.RIGHT);
	gb.setConstraints(c, gbc);
	p2.add(c);

	c = new TextField(sp.getConfString("quit_message"), 20);
	gbc.weightx = 1.0;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gb.setConstraints(c, gbc);
	p2.add(c);

	p.add(p2);
	components[EIRC.CONF_QUIT_MESSAGE] = c;

	/* Add tab to list. */
	String tab_tag = sp.getString("conf.tab.misc");
	page.add(p, tab_tag);
	tabs.add(tab_tag);

	/* Pre-connect settings Tab. */
	p = new Panel();
	p.setLayout(new GridLayout(0, 2, 2, 2));

	/* Request MOTD option. */
	c = new Checkbox(sp.getString("conf.request_motd"), sp.getConfBoolean("request_motd"));
	p.add(c);
	components[EIRC.CONF_REQUEST_MOTD] = c;

	/* Password option. */
	p2 = new Panel();
	gb = new GridBagLayout();
	gbc = new GridBagConstraints();
	p2.setLayout(gb);

	c = new Label(sp.getString("conf.password"), Label.RIGHT);
	gb.setConstraints(c, gbc);
	p2.add(c);

	c = new TextField(sp.getConfString("password"), 10);
	((TextField) c).setEchoChar('*');
	gbc.weightx = 1.0;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gb.setConstraints(c, gbc);
	p2.add(c);

	p.add(p2);
	components[EIRC.CONF_PASSWORD] = c;

	/* Add tab to list. */
	tab_tag = sp.getString("conf.tab.pre_connect");
	page.add(p, tab_tag);
	tabs.add(tab_tag);
    }

    void updateConfiguration() {
	for (int i = 0; i < EIRC.CONF_OPTIONS_COUNT; i++) {
	    Component c = components[i];
	    // Skip non-user configurable options.
	    if (null == c) {
		continue;
	    }

	    String tag = default_options[i][0];
	    if (c instanceof Checkbox) {
		sp.setConfBoolean(tag, ((Checkbox) c).getState());
	    } else if (c instanceof TextField) {
		sp.setConfString(tag, ((TextField) c).getText());
	    }
	}
    }

    public void setBackground(Color c) {
	super.setBackground(c);
	tabs.setBackground(c);
    }

    public void setForeground(Color c) {
	super.setForeground(c);
	tabs.setForeground(c);
    }
}
