/*
Eteria IRC Client, an RFC 1459 compliant client program written in Java.
Copyright (C) 2000  Javier Kohen <jkohen at tough.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import htmllayout.HtmlLayout;

class ControlPanel extends Panel implements MouseListener, ActionListener {
    private ActionListener actionListener;
    private MouseListener mouseListener;

    private ControlPanelClient cpc;

    private Hashtable components;

    ControlPanel(ControlPanelClient cpc, String html_layout) {
	this.cpc = cpc;

	setLayout(new HtmlLayout(html_layout));

	Component[] comps = new Component[5];   // Changed from 8 to 5  / jonas

	comps[0] = new ImageCanvas(cpc.getImage("status.nick"));
	comps[0].setName("logo");

	ImageButton ib;

	comps[1] = ib = new ImageButton();
	ib.setName("connect");
	// This component is initialized later.

	comps[2] = ib = new ImageButton();
	ib.setName("channel-list");
	ib.setImage(ImageButton.DISABLED,
		    cpc.getImage("status.channel-list.disabled"));
	ib.setImage(ImageButton.ENTERED,
		    cpc.getImage("status.channel-list.entered"));
	ib.setImage(ImageButton.EXITED,
		    cpc.getImage("status.channel-list.exited"));
	ib.setImage(ImageButton.PRESSED,
		    cpc.getImage("status.channel-list.pressed"));

	comps[3] = ib = new ImageButton();
	ib.setName("new-channel");
	ib.setImage(ImageButton.DISABLED,
		    cpc.getImage("status.new-channel.disabled"));
	ib.setImage(ImageButton.ENTERED,
		    cpc.getImage("status.new-channel.entered"));
	ib.setImage(ImageButton.EXITED,
		    cpc.getImage("status.new-channel.exited"));
	ib.setImage(ImageButton.PRESSED,
		    cpc.getImage("status.new-channel.pressed"));

	comps[4] = new RFC1459TextField();
	comps[4].setName("nick-entry");

//  	comps[5] = ib = new ImageButton();
//  	ib.setName("configuration");
//  	ib.setImage(ImageButton.ENTERED,
//  		    cpc.getImage("status.configuration.entered"));
//  	ib.setImage(ImageButton.EXITED,
//  		    cpc.getImage("status.configuration.exited"));
//  	ib.setImage(ImageButton.PRESSED,
//  		    cpc.getImage("status.configuration.pressed"));

//  	comps[6] = ib = new ImageButton();
//  	ib.setName("change-font");
//  	ib.setImage(ImageButton.DISABLED,
//  		    cpc.getImage("status.change-font.disabled"));
//  	ib.setImage(ImageButton.ENTERED,
//  		    cpc.getImage("status.change-font.entered"));
//  	ib.setImage(ImageButton.EXITED,
//  		    cpc.getImage("status.change-font.exited"));
//  	ib.setImage(ImageButton.PRESSED,
//  		    cpc.getImage("status.change-font.pressed"));
//      	ib.setEnabled(false); // WORKAROUND: #30 font changing is broken in com.lyrisoft.awt.*.

//  	comps[7] = ib = new ImageButton();
//  	ib.setName("nickserv");
//  	ib.setImage(ImageButton.DISABLED,
//  		    cpc.getImage("status.nickserv.disabled"));
//  	ib.setImage(ImageButton.ENTERED,
//  		    cpc.getImage("status.nickserv.entered"));
//  	ib.setImage(ImageButton.EXITED,
//  		    cpc.getImage("status.nickserv.exited"));
//  	ib.setImage(ImageButton.PRESSED,
//  		    cpc.getImage("status.nickserv.pressed"));

	components = new Hashtable(comps.length, 1.0f);

	for (int i = 0; i < comps.length; i++) {
	    Component c = comps[i];

	    components.put(c.getName(), c);
	    add(c, c.getName());

	    c.addMouseListener(this);
	    if (c instanceof TextField) {
		((TextField) c).addActionListener(this);
	    }
	}

	setConnected(false);
    }

    public void setBackground(Color c) {
	super.setBackground(c);
	for (Enumeration e = components.elements(); e.hasMoreElements(); ) {
	    Component comp = (Component) e.nextElement();
	    comp.setBackground(c);
	}
    }

    public void setForeground(Color c) {
	super.setForeground(c);
	for (Enumeration e = components.elements(); e.hasMoreElements(); ) {
	    Component comp = (Component) e.nextElement();
	    comp.setForeground(c);
	}
    }

    public void setTextBackground(Color c) {
	Component comp = (Component) components.get("nick-entry");
	comp.setBackground(c);
    }

    public void setTextForeground(Color c) {
	Component comp = (Component) components.get("nick-entry");
  	comp.setForeground(c);
    }

    public void addMouseListener(MouseListener l) {
	if (null != l) {
	    mouseListener = AWTEventMulticaster.add(mouseListener, l);
	}
    }

    public void removeMouseListener(MouseListener l) {
	if (null != l) {
	    mouseListener = AWTEventMulticaster.remove(mouseListener, l);
	}
    }

    public void mouseClicked(MouseEvent e) {
	if (mouseListener != null) {
	    mouseListener.mouseClicked(e);
	}
    }

    public void mousePressed(MouseEvent e) {
	if (mouseListener != null) {
	    mouseListener.mousePressed(e);
	}
    }

    public void mouseReleased(MouseEvent e) {
	if (mouseListener != null) {
	    mouseListener.mouseReleased(e);
	}
    }

    public void mouseEntered(MouseEvent e) {
	if (mouseListener != null) {
	    mouseListener.mouseEntered(e);
	}
    }

    public void mouseExited(MouseEvent e) {
	if (mouseListener != null) {
	    mouseListener.mouseExited(e);
	}
    }

    public void addActionListener(ActionListener l) {
	if (null != l) {
	    actionListener = AWTEventMulticaster.add(actionListener, l);
	}
    }

    public void removeActionListener(ActionListener l) {
	if (null != l) {
	    actionListener = AWTEventMulticaster.remove(actionListener, l);
	}
    }

    public void actionPerformed(ActionEvent e) {
	if (actionListener != null) {
	    actionListener.actionPerformed(e);
	}
    }

    public void setEnabled(String name, boolean state) {
	((Component) components.get(name)).setEnabled(state);
    }
    
    public String getNick() {
	RFC1459TextField t = (RFC1459TextField) components.get("nick-entry");
	return t.getText();
    }
    
    public void setNick(String new_nick) {
	RFC1459TextField t = (RFC1459TextField) components.get("nick-entry");
	t.setText(new_nick);
    }

    public void requestFocus() {
	RFC1459TextField t = (RFC1459TextField) components.get("nick-entry");
	t.requestFocus();
    }

    public void setConnected(boolean connected) {
	String tree = connected ? "disconnect" : "connect";
	ImageButton ib = (ImageButton) components.get("connect");

	ib.setImage(ImageButton.ENTERED,
		    cpc.getImage("status." + tree + ".entered"));
	ib.setImage(ImageButton.EXITED,
		    cpc.getImage("status." + tree + ".exited"));
	ib.setImage(ImageButton.PRESSED,
		    cpc.getImage("status." + tree + ".pressed"));

	// If we omit this, the button disappears in MSIE when the connection fails...
	validate();

	ib.repaint();
    }
}
