/*
Eteria IRC Client, an RFC 1459 compliant client program written in Java.
Copyright (C) 2000  Javier Kohen <jkohen at tough.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.reflect.*;
import java.net.*;
import java.text.ChoiceFormat;
import java.text.Collator;
import java.text.MessageFormat;
import java.text.ParseException;
import java.util.*;

public class EIRC extends Applet implements ServerProcess, ControlPanelClient, MouseListener, ActionListener, ChatPanelListener {
    public final static String PACKAGE = "Eteria IRC Client";
    public final static String VERSION = "20001229";

    // Localy defined for paranormal.se
    public final static String SERVER = "irc.paranormal.se";
    public final static String PORT = "6667";

    static String PRE_ERROR;
    static String PRE_ACTION;
    static String PRE_INFO;
    static String PRE_UNMANGLED;

    // These items have to follow the order of the list below.
    public static final int CONF_SPECIAL_SERVICES = 0;
    public static final int CONF_HIDEIP = 1;
    public static final int CONF_PREFIX_OWN = 2;
    public static final int CONF_SEE_EVERYTHING_FROM_SERVER = 3;
    public static final int CONF_SEE_JOIN = 4;
    public static final int CONF_LIST_PARAMETER = 5;
    public static final int CONF_REQUEST_MOTD = 6;
    public static final int CONF_FILTER_MIRC_ATTRIBS = 7;
    public static final int CONF_SERVICES_PASSWORD = 8;
    public static final int CONF_SERVICES_EMAIL = 9;
    public static final int CONF_QUIT_MESSAGE = 10;
    public static final int CONF_PASSWORD = 11;
    public static final int CONF_DEBUG_TRAFFIC = 12;
    public static final int CONF_HAVE_HALFOPS = 13;
    public static final int CONF_SERVICE_BOTS = 14;
    public static final int CONF_WINDOW_WIDTH = 15;
    public static final int CONF_WINDOW_HEIGHT = 16;
    public static final int CONF_OPTIONS_COUNT = 17;
    // "Name", "default value".
    final static String[][] default_options = {
	{ "special_services", "false" },
	{ "hideip", "false" },
	{ "prefix_own", "false" },
	{ "see_everything_from_server", "false" },
	{ "see_join", "true" },
	{ "list_parameter", "" },
	{ "request_motd", "false" },
	{ "filter_mirc_attribs", "true" },
	{ "services_password", "" },
	{ "services_email", "" },
	{ "quit_message", "Please, configure me." },
	{ "password", "" },
	{ "debug_traffic", "false" },
	{ "have_halfops", "true" },
	{ "service_bots", "" },
	{ "window_width", "400" },
	{ "window_height", "300" },
    };

    final static String param_info[][] = {
	{ "server", "string", "IRC server's address" },
	{ "port", "0-65535", "IRC server's port" },
	{ "mainbg", "color", "general background color" },
	{ "mainfg", "color", "general foreground color" },
	{ "textbg", "color", "text background color" },
	{ "textfg", "color", "text foreground color" },
	{ "channel", "string", "channel to auto-join" },
	{ "titleExtra", "string", "extra text for windows' titles" },
	{ "username", "string", "username part of hostmask" },
	{ "realname", "string", "user's real name" },
	{ "nickname", "RFC 1459 string", "user's nick name" },
	{ "password", "string", "user's password to server" },
	{ "login", "0-1", "whether to try auto-login" },
	{ "language", "two-letter ISO-639 Language Code", "user's language code" },
	{ "country", "two-letter ISO-3166 Country Code", "user's country code" },
    };

    static Command user_commands[];
    static Hashtable commands;

    Collator collator;
    ConfigurationBundle config;
    CollatedHashtable channels;
    CollatedHashtable privates;
    Hashtable ignore_list;

    ResourceBundle resources;
    ResourceBundle images;

    Color mainbg, mainfg, textbg, textfg;

    ControlPanel control_panel;
    private ChatPanelContainer chat_panel;
    private StatusWindow status;

    String current_nick;
    String new_nick;
    Vector chan_list;
    Socket s;
    ClientThread ct;

    String default_channel;
    boolean connected = false;
    boolean logged_in = false;
    boolean special_services;
    boolean got_default_channel_in_list = false;
    boolean debug_traffic;

    // Dialogs.
    ChannelList channel_list;
    ChangeFont change_font;
    NickServCommander nickserv_commander;
    Configurator configurator;
    NewChannel new_channel;

    static {
	// You have to add the commands, but not the aliases, to the invokeCommand method, too.
	user_commands = new Command[20];
	user_commands[0] = new Command("help", 0, 1);
	user_commands[1] = new Command("quote", 1, 1);
	user_commands[2] = new Command("join", 1, 1);
	user_commands[3] = new Command("j", user_commands[2]);
	user_commands[4] = new Command("part", 0, 2);
	user_commands[5] = new Command("ping", 1, 1);
	user_commands[6] = new Command("quit", 0, 1);
	user_commands[7] = new Command("nick", 1, 1);
	user_commands[8] = new Command("me", 1, 1);
	user_commands[9] = new Command("query", 1, 2);
	user_commands[10] = new Command("q", user_commands[9]);
	user_commands[11] = new Command("msg", 2, 2);
	user_commands[12] = new Command("eirc", 0, 0);
	user_commands[13] = new Command("kban", 1, 2);
	user_commands[14] = new Command("ignore", 1, 1);
	user_commands[15] = new Command("unignore", 1, 1);
	user_commands[16] = new Command("server", 0, 0);

        commands = new Hashtable(50, 1);
	commands.put("ping", new Integer(-1));
	commands.put("nick", new Integer(-2));
	commands.put("join", new Integer(-3));
	commands.put("mode", new Integer(-4));
	commands.put("part", new Integer(-5));
	commands.put("quit", new Integer(-6));
	commands.put("kick", new Integer(-7));
	commands.put("topic", new Integer(-8));
	commands.put("privmsg", new Integer(-9));
	commands.put("notice", new Integer(-10));
	commands.put("error", new Integer(-11));
	commands.put("307", new Integer(307));
  	commands.put("310", new Integer(310));
  	commands.put("311", new Integer(311));
	commands.put("312", new Integer(312));
  	commands.put("313", new Integer(313));
  	commands.put("317", new Integer(317));
	commands.put("318", new Integer(318)); // no action needed.
	commands.put("319", new Integer(319));
  	commands.put("321", new Integer(321));
	commands.put("322", new Integer(322));
  	commands.put("323", new Integer(323));
  	commands.put("328", new Integer(328));
	commands.put("331", new Integer(331));
	commands.put("332", new Integer(332));
  	commands.put("333", new Integer(333));
  	commands.put("353", new Integer(353));
	commands.put("366", new Integer(366));
	commands.put("372", new Integer(372));
	commands.put("375", new Integer(375));
    	commands.put("376", new Integer(376));
    	commands.put("401", new Integer(401));
	commands.put("422", new Integer(422));
  	commands.put("432", new Integer(432));
  	commands.put("433", new Integer(433));
  	commands.put("438", new Integer(438));
  	commands.put("464", new Integer(464));
  	commands.put("471", new Integer(471));
  	commands.put("473", new Integer(473));
  	commands.put("474", new Integer(474));
  	commands.put("475", new Integer(475));
  	commands.put("482", new Integer(482));
    }

    public void init() {
	collator = RFC1459.getCollator();
	channels = new CollatedHashtable(collator);
	privates = new CollatedHashtable(collator);
	config = new ConfigurationBundle(default_options, "configuration");
	images = ResourceBundle.getBundle("Images");
	ignore_list = new Hashtable();

	String t;
	String t2[] = new String[2];
	if (null != (t2[0] = getParameter("language")) &&
	    null != (t2[1] = getParameter("country"))) {
	    // Use user specified locale for resources
	    resources = ResourceBundle.getBundle("Resources", new Locale(t2[0], t2[1]));	    
	} else {
	    // Use default locale for resources
	    resources = ResourceBundle.getBundle("Resources");
	}

	PRE_ERROR = getString("eirc.pre.error");
	PRE_ACTION = getString("eirc.pre.action");
	PRE_INFO = getString("eirc.pre.info");
	PRE_UNMANGLED = getString("eirc.pre.unmangled");

	special_services = getConfBoolean("special_services");
	debug_traffic = getConfBoolean("debug_traffic");
	default_channel = getParameter("channel");

	mainbg = SystemColor.window;
	if (null != (t = getParameter("mainbg"))) {
	    mainbg = Color.decode(t);
	}
	mainfg = SystemColor.windowText;
	if (null != (t = getParameter("mainfg"))) {
	    mainfg = Color.decode(t);
	}
	textbg = SystemColor.text;
	if (null != (t = getParameter("textbg"))) {
	    textbg = Color.decode(t);
	}
	textfg = SystemColor.textText;
	if (null != (t = getParameter("textfg"))) {
	    textfg = Color.decode(t);
	}

	if (special_services) {
	    if (null != (t = getParameter("servPassword"))) {
		setConfString("services_password", t);
	    }
	    if (null != (t = getParameter("servEmail"))) {
		setConfString("services_email", t);
	    }
	}

	GridBagLayout gb = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();

	setLayout(gb);

	gbc.weightx = 1.0;
	gbc.insets = new Insets(2, 2, 2, 2);

	String html_layout;

	try {
  	    html_layout = new String(Resource.getResource("ControlPanel.html", this));
	} catch (IOException e) {
	    System.err.println(e);
	    return;
	}

	control_panel = new ControlPanel(this, html_layout);
	control_panel.setEnabled("channel-list", false);
	control_panel.setEnabled("new-channel", false);
	//	control_panel.setEnabled("nickserv", false);   // jonas

	gb.setConstraints(control_panel, gbc);
	add(control_panel);

	gbc.gridy = 1;

	chat_panel = new ChatPanelContainer(this);
	chat_panel.setRevertTo(default_channel);
	gbc.weighty = 1.0;
	gbc.fill = GridBagConstraints.BOTH;
	gb.setConstraints(chat_panel, gbc);
	add(chat_panel);

	status = new StatusWindow(this, getString("eirc.status"));
	status.setFont(OutputWindow.DEFAULT_FONT);
	status.setBackground(mainbg);
	status.setForeground(mainfg);
	status.setTextBackground(textbg);
	status.setTextForeground(textfg);
  	status.requestFocus();

  	chat_panel.addStatus(status, getString("eirc.status"));

	validate();

	setBackground(mainbg);
	setForeground(mainfg);
	setTextBackground(textbg);
	setTextForeground(textfg);

	control_panel.addMouseListener(this);
	control_panel.addActionListener(this);

	if (null != (t = getParameter("nickname"))) {
	    char str[] = t.toCharArray();
	    char numbers[] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9' };

	    // Replace every '?' occurrence by a random number.
	    for (int i = 0; i < str.length; i++) {
		if ('?' == str[i]) {
		    str[i] = numbers[(int) Math.floor(Math.random() * 10.0)];
		}
	    }
	    control_panel.setNick(String.copyValueOf(str));
	}

	if (null != (t = getParameter("password"))) {
	    setConfString("password", t);
	}

	// See if auto-login was requested.
	if (null != (t = getParameter("login")) && !t.equals("0")) {
	    // Can't login if no nickname was provided.
	    if (null == (t = getParameter("nickname")) || t.equals("")) {
		status.printError(getString("eirc.s26"));
	    } else {
		try {
		    connect();
		} catch (java.io.IOException e) {
  		    System.err.println(e);
		}
	    }
	}
    }

    public String getAppletInfo() {
	return (EIRC.PACKAGE + " " + EIRC.VERSION + ", an RFC 1459 compliant client program written in Java.\n" +
		"Copyright (C) 2000  Javier Kohen <jkohen at tough.com>");
    }

    public String[][] getParameterInfo() {
	return param_info;
    }

    public void destroy() {
	disconnect();

	// Let's close all open privates
	for (Enumeration e = privates.elements(); e.hasMoreElements(); ) {
	    PrivateWindow w = (PrivateWindow) e.nextElement();
	    w.dispose();
	}
	privates.clear();
    }

    synchronized void connect() throws IOException, UnknownHostException {
	String server_name = SERVER;

	InetAddress server;
	try {
	    server = InetAddress.getByName(server_name);
	} catch (UnknownHostException e) {
	    Object a[] = { server_name };
	    MessageFormat mf = new MessageFormat(getString("eirc.s2"));
	    status.printError(mf.format(a));

	    throw e;
	} catch (SecurityException e) {
	    Object a[] = { server_name };
	    MessageFormat mf = new MessageFormat(getString("eirc.s2"));
	    status.printError(mf.format(a));

	    throw e;
	}

	int port = Integer.parseInt(PORT);

	try {
	    s = new Socket(server, port);
	} catch (IOException e) {
	    Object a[] = { server.getHostName(), new Integer(port) };
	    MessageFormat mf = new MessageFormat(getString("eirc.s3"));
	    status.printError(mf.format(a));

	    throw e;
	}

	control_panel.setEnabled("channel-list", true);
	control_panel.setEnabled("new-channel", true);
	// jonas
	//	if (special_services) {
	//	    control_panel.setEnabled("nickserv", true);
	//	}

	ct = new ClientThread(this, s);
	ct.start();
	connected = true;

	control_panel.setConnected(true);

	login();
    }

    void login() {
	String password = getConfString("password");
	if (!password.equals("")) {
	    String p[] = { password };
	    sendMessage("pass", p);
	}
	{
	    new_nick = control_panel.getNick();
	    String p[] = { new_nick };
	    sendMessage("nick", p);
	}

	String t;
	String username = "eirc";
	String realname = "Eteria IRC Client";

	if (null != (t = getParameter("username")) && !t.equals("")) {
	    username = t;
	}
	if (null != (t = getParameter("realname"))) {
	    realname = t;
	}
	{
	    String p[] = { username, "0", "0", realname };
	    sendMessage("user", p);
	}
    }

    void pos_login() {
	current_nick = new_nick;
	status.requestFocus();

	if (getConfBoolean("request_motd")) {
	    String p[] = { };
	    sendMessage("motd", p);
	}
	if (null != default_channel && !default_channel.equals("")) {
	    String p[] = { default_channel };
	    sendMessage("join", p);
	}

	String services_password = getConfString("services_password");
	if (!services_password.equals("")) {
	    String p[] = { "nickserv", "identify " + services_password };
	    sendMessage("privmsg", p);
	}
    }

    public synchronized void disconnect() {
	if (!connected) {
	    return;
	}

	closeChannelList();
	// Let's close all open channels
	for (Enumeration e = channels.elements(); e.hasMoreElements(); ) {
	    ChannelWindow w = (ChannelWindow) e.nextElement();
	    w.dispose();
	}
	channels.clear();

	if (logged_in) {
	    String p[] = { getConfString("quit_message") };
	    sendMessage("quit", p);
	}
	ct.disconnect();

  	connected = false;
	logged_in = false;
	control_panel.setEnabled("channel-list", false);
	control_panel.setEnabled("new-channel", false);
	//	control_panel.setEnabled("nickserv", false);  // jonas

	try {
	    s.close();
	} catch (IOException e) {
	    System.err.println(e);
	}

	control_panel.setConnected(false);
    }

    public void processMessage(RemoteMessage m) {
	m.setFilterMIRCAttribs(getConfBoolean("filter_mirc_attribs"));

	String command_key = m.getCommand().toLowerCase();
	String prefix = m.getPrefix();
	String params[] = m.getParameters();
        Integer t_cmd = (Integer) commands.get(command_key);

	{
	    int endOff = prefix.indexOf('!');
	    if (-1 != endOff) {
		prefix = prefix.substring(0, endOff);
	    }
	}

	if (debug_traffic || null == t_cmd) {
	    // Strip CRLF from the end of the message.
	    String t = m.toString();
	    t = t.substring(0, t.length() - 2);

	    if (debug_traffic) {
		System.out.println("< ".concat(t));
	    }

	    if (null == t_cmd) {
		if (getConfBoolean("see_everything_from_server")) {
		    String s;

		    // If it's a numeric IRC command, take the crud out of it.
		    try {
			Integer.parseInt(command_key); // We don't care about the result, we just want to know if it's a number.
			s = params[params.length - 1];
		    } catch (NumberFormatException e) {
			s = t;
		    }

		    status.output(PRE_UNMANGLED.concat(s).concat("\n"));
		}
		return;
	    }
	}

	int command = t_cmd.intValue();
	boolean is_prefix_ignored = ignore_list.containsKey(prefix);

	switch (command) {
	case -1: { // PING
	    String p[] = { params[0] };
	    sendMessage("pong", p);
	    break;
	}
	case -2: { // NICK
	    if (prefix.equals(current_nick)) { // Self nick change
		current_nick = params[0];
		control_panel.setNick(current_nick);

		Object a[] = { current_nick };
		MessageFormat mf = new MessageFormat(getString("eirc.s4"));
		status.printInfo(mf.format(a));
	    } else { // Other's nick change
		{
		    Object a[] = { prefix, params[0] };
		    MessageFormat mf = new MessageFormat(getString("eirc.s5"));
		    status.printInfo(mf.format(a));
		}
	    }

	    // Update private window (if there's one) accordingly
	    if (privates.containsKey(prefix)) {
		PrivateWindow pw = (PrivateWindow) privates.get(prefix);
		chat_panel.renameChat(prefix, params[0]);
		privates.remove(prefix);
		privates.put(params[0], pw);
	    }

	    // Update channel window accordingly
	    for (Enumeration e = channels.elements(); e.hasMoreElements(); ) {
		ChannelWindow w = (ChannelWindow) e.nextElement();
		w.changeNick(prefix, params[0]);
	    }
	    break;
	}
	case -3: { // JOIN
	    // If it's me, just open the channel window.
	    if (current_nick.equals(prefix)) {
		openChannel(params[0]);
	    } else {
		ChannelWindow cw = getChannel(params[0]);
		cw.addNick(prefix);
		if (getConfBoolean("see_join")) {
		    Object a[] = { prefix };
		    MessageFormat mf = new MessageFormat(getString("eirc.s6"));
		    cw.printInfo(mf.format(a));
		}
	    }
	    break;
	}
	case -4: { // MODE
	    ChannelWindow cw = getChannel(params[0]);
	    boolean sign = true;
	    int j = 2;
	    char modes[] = params[1].toCharArray();

	    for (int i = 0; i < modes.length; i++) {
		String msg_tag = null;

		switch (modes[i]) {
		case '+':
		    sign = true;
		    break;
		case '-':
		    sign = false;
		    break;
		case 'v':
		    msg_tag = sign ? "eirc.voice" : "eirc.unvoice";
		    break;
		case 'o':
		    msg_tag = sign ? "eirc.op" : "eirc.deop";
		    break;
		case 'h':
		    msg_tag = sign ? "eirc.hop" : "eirc.dehop";
		    break;
		case 'b':
		    msg_tag = sign ? "eirc.ban" : "eirc.unban";
		    break;
		}

		if (null != msg_tag) {
		    MessageFormat mf = new MessageFormat(getString(msg_tag));

		    // Remove possible hostmask.
		    int endOfNick;
		    String nick = params[j];
		    if (-1 != (endOfNick = nick.indexOf('!'))) {
			nick = nick.substring(0, endOfNick);
		    }

		    cw.changeMode(nick, modes[i], sign);

		    Object a[] = { prefix, nick };
		    cw.printInfo(mf.format(a));

		    j++;
		}
	    }
	    break;
	}
	case -5: { // PART
	    ChannelWindow cw = getChannel(params[0]);
	    if (current_nick.equals(prefix)) {
		if (getConfBoolean("see_join")) {
		    Object a[] = { params[0] };
		    MessageFormat mf = new MessageFormat(getString("eirc.s7"));
		    status.printInfo(mf.format(a));
		}
		channels.remove(params[0]);
	    } else {
		cw.removeNick(prefix);
		if (getConfBoolean("see_join")) {
		    Object a[] = { prefix };
		    MessageFormat mf = new MessageFormat(getString("eirc.s8"));
		    cw.printInfo(mf.format(a));
		}
	    }
	    break;
	}
	case -6: { // QUIT
	    for (Enumeration e = channels.elements(); e.hasMoreElements(); ) {
		ChannelWindow w = (ChannelWindow) e.nextElement();
		w.removeNick(prefix);
	    }

	    Object a[] = { prefix, params[0] };
	    MessageFormat mf = new MessageFormat(getString("eirc.s9"));
	    status.printInfo(mf.format(a));
	    break;
	}
	case -7: { // KICK
	    ChannelWindow cw = getChannel(params[0]);
	    String reason = null != params[2] ? params[2] : "";

	    if (current_nick.equals(params[1])) {
		Object a[] = { params[0], prefix, reason };
		MessageFormat mf = new MessageFormat(getString("eirc.s10"));
		status.printInfo(mf.format(a));

		chat_panel.removeChat(params[0]);
		cw.dispose();
		channels.remove(params[0]);
	    } else {
		Object a[] = { params[1], prefix, reason };
		MessageFormat mf = new MessageFormat(getString("eirc.s11"));
		status.printInfo(mf.format(a));

		cw.removeNick(params[1]);
	    }
	    break;
	}
	case -8: { // TOPIC
	    ChannelWindow cw = getChannel(params[0]);
	    cw.setTopic(params[1]);

	    Object a[] = { prefix, params[1] };
	    MessageFormat mf = new MessageFormat(getString("eirc.s36"));
	    cw.printInfo(mf.format(a));
	    break;
	}
	case -9: { // PRIVMSG
	    // If the source of this message is in the ignore list, ignore this message.
	    if (is_prefix_ignored) {
		break;
	    }

	    boolean isChannel = Channel.isChannel(params[0]);

	    char ch = params[1].charAt(0);
	    if ('\001' == ch) { // CTCP query
		boolean hasValue = true;
		int endOff = params[1].indexOf(" ");
		if (-1 == endOff) {
		    endOff = params[1].length() - 1;
		    hasValue = false;
		}
		String query = params[1].substring(1, endOff).toLowerCase();
		String value = hasValue ? params[1].substring(endOff + 1, params[1].length()) : "";

		if (query.equalsIgnoreCase("action")) {
		    ChatWindow target = isChannel ?
			(ChatWindow) getChannel(params[0]) :
			(ChatWindow) openPrivate(prefix);
		    target.printAction(value, prefix);
		} else if (query.equalsIgnoreCase("ping")) {
		    String p[] = { prefix, "\001PING " + value + "\001" };
		    sendMessage("notice", p);
		}
	    } else {
		ChatWindow target = isChannel ? (ChatWindow) getChannel(params[0]) : (ChatWindow) openPrivate(prefix);
		target.printPrivmsg(params[1], prefix);
	    }
	    break;
	}
	case -10: { // NOTICE
	    // If the source of this message is in the ignore list, ignore this message.
	    if (is_prefix_ignored) {
		break;
	    }

	    if (!prefix.equals("")) {
		int endOff = prefix.indexOf('!');
		if (-1 != endOff) {
		    prefix = prefix.substring(0, endOff);
		}
	    }

	    if (Channel.isChannel(params[0])) {
		getChannel(params[0]).printNotice(params[1], prefix);
	    } else if (special_services &&
		       !prefix.equals("") &&
		       -1 != getConfString("service_bots").indexOf(prefix)) {
		// Output notices from service bots to a dedicated (per bot) window.
		openPrivate(prefix).printNotice(params[1], prefix);
	    } else if (params[0].equals(current_nick)) {
		char ch = params[1].charAt(0);
		if ('\001' == ch) { // CTCP response
		    boolean hasValue = true;
		    int endOff = params[1].indexOf(" ");
		    if (-1 == endOff) {
			endOff = params[1].length() - 1;
			hasValue = false;
		    }
		    String query = params[1].substring(1, endOff).toLowerCase();
		    String value = hasValue ? params[1].substring(endOff + 1, params[1].length() - 2) : "";


		    if (query.equalsIgnoreCase("ping")) {
			double diff = Double.NEGATIVE_INFINITY;

			if (hasValue) {
			    try {
				long launch = Long.parseLong(value);
				long arrive = (new Date()).getTime();
				diff = (arrive - launch) / 1000.0;
			    } catch (NumberFormatException e) {
				// No action needed.
				hasValue = false;
			    }
			}

			{
			    Object a[] = { prefix, new Double(diff) };
			    MessageFormat mf = new MessageFormat(getString("eirc.s12"));
			    double limits[] = { ChoiceFormat.previousDouble(0.0), 0.0, 1.0, ChoiceFormat.nextDouble(1.0) };
			    String times[] = {
				getString("eirc.s12.0"),
				getString("eirc.s12.1"),
				getString("eirc.s12.2"),
				getString("eirc.s12.3")
			    };
			    mf.setFormat(1, new ChoiceFormat(limits, times));
			    status.printInfo(mf.format(a));
			}
		    }
		} else {
		    status.printNotice(params[1], prefix);
		}
	    } else {
		status.printNotice(params[0] + " " + params[1], prefix);
	    }
	    break;
	}
	case -11: { // ERROR
	    status.printError(params[0]);
	    break;
	}
	case 307: { // RPL_WHOISREGISTERED (DalNet)
	    Object a[] = { params[1] };
	    MessageFormat mf = new MessageFormat(getString("eirc.s27"));
	    status.printInfo(mf.format(a));
	    break;
	}
	case 310: { // RPL_WHOISOPSTATUS
	    Object a[] = { params[1], params[2] };
	    MessageFormat mf = new MessageFormat(getString("eirc.310"));
	    status.printInfo(mf.format(a));
	    break;
	}
	case 311: { // RPL_WHOISUSER
	    String t;
	    if (getConfBoolean("hideip")) {
		params[3] = "xxx";
	    }

	    {
		Object a[] = { params[1], params[2], params[3] };
		MessageFormat mf = new MessageFormat(getString("eirc.s13"));
		status.printInfo(mf.format(a));
	    }
	    {
		Object a[] = { params[1], params[5] };
		MessageFormat mf = new MessageFormat(getString("eirc.s14"));
		status.printInfo(mf.format(a));
	    }
	    break;
	}
	case 312: { // RPL_WHOISSERVER
	    Object a[] = { params[1], params[2], params[3] };
	    MessageFormat mf = new MessageFormat(getString("eirc.s15"));
	    status.printInfo(mf.format(a));
	    break;
	}
	case 313: { // RPL_WHOISOPERATOR
	    Object a[] = { params[1] };
	    MessageFormat mf = new MessageFormat(getString("eirc.s16"));
	    status.printInfo(mf.format(a));
	    break;
	}
	case 317: { // RPL_WHOISIDLE
	    Object a[] = { params[1], new Integer(params[2]) };
	    MessageFormat mf = new MessageFormat(getString("eirc.s17"));
	    double limits[] = { 0, 1, 2 };
	    String times[] = {
		getString("eirc.s17.0"),
		getString("eirc.s17.1"),
		getString("eirc.s17.2")
	    };
	    mf.setFormat(1, new ChoiceFormat(limits, times));
	    status.printInfo(mf.format(a));
	    break;
	}
	case 319: { // RPL_WHOISCHANNELS
	    Object a[] = { params[1], params[2] };
	    MessageFormat mf = new MessageFormat(getString("eirc.s18"));
	    status.printInfo(mf.format(a));
	    break;
	}
	case 321: { // RPL_LISTSTART
	    chan_list = new Vector(5, 5);
	    if (null == default_channel || default_channel.equals("")) {
		got_default_channel_in_list = true;
	    } else {
		got_default_channel_in_list = false;
	    }
	    break;
	}
	case 322: { // RPL_LIST
	    Channel new_chan;

	    if (!got_default_channel_in_list && params[1].equalsIgnoreCase(default_channel)) {
		got_default_channel_in_list = true;

		// Put this channel topmost in the list.
		new_chan = new Channel(params[1], params[2], params[3], 0);
	    } else {
		new_chan = new Channel(params[1], params[2], params[3]);
	    }

	    chan_list.addElement(new_chan);
	    break;
	}
	case 323: { // RPL_LISTEND
	    if (!got_default_channel_in_list) {
		// Put this channel topmost in the list.
		chan_list.addElement(new Channel(default_channel, "0", "", 0));
	    }

	    if (0 < chan_list.size()) {
		openChannelList();
	    } else {
		status.printInfo(getString("eirc.s19"));
	    }
	    break;
	}
	case 328: { // RPL_CHANURL
	    ChannelWindow ow = getChannel(params[1]);
	    if (null == ow) {
		break;
	    }

	    Object a[] = { params[0], params[2] };
	    MessageFormat mf = new MessageFormat(getString("eirc.328"));
	    ow.printInfo(mf.format(a));
	    break;
	}
	case 331: { // RPL_NOTOPIC
	    OutputWindow ow = getChannel(params[1]);
	    if (null == ow) {
		ow = status;
	    } else {
		((ChannelWindow) ow).setTopic("");
	    }

	    Object a[] = { params[1] };
	    MessageFormat mf = new MessageFormat(getString("eirc.s37"));
	    ow.printInfo(mf.format(a));
	    break;
	}
	case 332: { // RPL_TOPIC
	    OutputWindow ow = getChannel(params[1]);
	    if (null == ow) {
		ow = status;
	    } else {
		((ChannelWindow) ow).setTopic(params[2]);
	    }

	    Object a[] = { params[1], params[2] };
	    MessageFormat mf = new MessageFormat(getString("eirc.s38"));
	    ow.printInfo(mf.format(a));
	    break;
	}
	case 333: { // RPL_TOPICINFO
	    OutputWindow ow = getChannel(params[1]);
	    if (null == ow) {
		ow = status;
	    }

	    // We're sent the time in seconds, but Date expects millis.
	    Date topic_date = new Date(Long.parseLong(params[3]) * 1000);

	    Object a[] = { params[2], topic_date, topic_date };
	    MessageFormat mf = new MessageFormat(getString("eirc.s35"));
	    ow.printInfo(mf.format(a));
	    break;
	}
	case 353: { // RPL_NAMREPLY
	    ChannelWindow cw = getChannel(params[2]);
	    StringTokenizer st = new StringTokenizer(params[3], " ");
	    int tokens = st.countTokens();
	    String [] names = new String [tokens];

	    for (int i = 0; i < tokens; i++) {
		names[i] = st.nextToken();
	    }
	    cw.addNicks(names);
	    break;
	}
	case 372: { // RPL_MOTD
	    status.output(params[1].concat("\n"));
	    break;
	}
	case 376:   // RPL_ENDOFMOTD
	case 422: { // ERR_NOMOTD
	    // If we are here, it means we have registered
	    if (!logged_in) {
		logged_in = true;

		// See what the server thinks our nick is from the message it sent. Some servers might trim nicks to 9 chars.
		if (!new_nick.equals(params[0])) {
		    new_nick = params[0];
		    control_panel.setNick(new_nick);

		    Object a[] = { new_nick };
		    MessageFormat mf = new MessageFormat(getString("eirc.s4"));
		    status.printInfo(mf.format(a));
		}

		pos_login();
	    }
	    break;
	}
	case 432:   // ERR_ERRONEUSNICKNAME
	case 433: { // ERR_NICKNAMEINUSE
	    status.printError(getString("eirc." + command));
	    break;
	}
	case 438: { // ERR_NICKCHANGETOOFAST
	    status.printError(params[2]);
	    break;
	}
	case 464: { // ERR_PASSWDMISMATCH
	    status.printError(params[1]);
	    disconnect();
	    break;
	}
	case 401:   // ERR_NOSUCHNICK
	case 471:   // ERR_CHANNELISFULL
	case 473:   // ERR_INVITEONLYCHAN
	case 474:   // ERR_BANNEDFROMCHAN
	case 475: { // ERR_BADCHANNELKEY
	    Object a[] = { params[1] };
	    MessageFormat mf = new MessageFormat(getString("eirc." + command));
	    status.printError(mf.format(a));
	    break;
	}
	case 482: { // ERR_CHANOPRIVSNEEDED
	    getChannel(params[1]).printError(getString("eirc." + 482));
	    break;
	}
	}
    }

    public void sendCommand(String text, OutputWindow target) {
	StringTokenizer st = new StringTokenizer(text);

	if (!st.hasMoreTokens()) {
	    status.printError(getString("eirc.s20"));
	}

	Command c;
	String action = st.nextToken();
	if (null == (c = findCommand(action))) {
	    // Let the server decide if this command is correct.
	    String a[] = new String [0];
	    sendMessage(text, a);

	    return;
	}

	// Invoke getCommand() to resolve possible command aliases. Resolution is of one-level depth.
	action = c.getCommand();
	int required = c.getRequiredParameters();
	int accepted = c.getAcceptedParameters();
	String help = getString("help.".concat(action).concat(".short"));

	/* Parse parameters */

	Vector parameters = new Vector(accepted);
	int found = 0;
	while (found < Math.min(accepted, st.countTokens()) - 1) {
	    if (st.hasMoreTokens()) {
		parameters.addElement(st.nextToken());
		found++;
	    } else if (found < required) {
		Object a[] = { help };
		MessageFormat mf = new MessageFormat(getString("eirc.s22"));
		status.printError(mf.format(a));

		return;
	    }
	}

	// Add the last parameter, which may contain spaces.
	if (st.hasMoreTokens()) {
	    String tok = st.nextToken("");
	    parameters.addElement(tok.trim());
	    found++;
	}

	if (found < required) {
	    Object a[] = { help };
	    MessageFormat mf = new MessageFormat(getString("eirc.s22"));
	    status.printError(mf.format(a));
	    return;
	}

//  	try {
//	    invokeCommand(action, parameters, target);
//  	} catch (ClassNotFoundException e) {
//  	    System.err.println(e);
//  	} catch (NoSuchMethodException e) {
//  	    System.err.println(e);
//  	} catch (InvocationTargetException e) {
//  	    System.err.println(e);
//  	} catch (IllegalAccessException e) {
//  	    System.err.println(e);
//  	}

	invokeCommand(action, parameters, target);
    }

    Command findCommand(String action) {
	int hashCode = action.toLowerCase().hashCode();

	for (int i = 0; null != user_commands[i]; i++) {
	    if (user_commands[i].getHashCode() == hashCode) {
		return user_commands[i];
	    }
	}

	return null;
    }

    /* Netscape Navigator's security model didn't like this code. Its been replaced by the method below this comment, but the code is ugly.

       // Invoke a method of this class following the form: void <name>(Vector, OutputWindow).
       void invokeCommand(String name, Vector parameters, OutputWindow target) throws ClassNotFoundException, NoSuchMethodException, InvocationTargetException, IllegalAccessException {
       name = "cmd_" + name;
       Class c = Class.forName("EIRC");
       Class a[] = { Class.forName("java.util.Vector"),
       Class.forName("OutputWindow")
       };
       Method m = c.getDeclaredMethod(name, a);
       Object o[] = { parameters, target };
       m.invoke(this, o);
    */

    void invokeCommand(String name, Vector parameters, OutputWindow target) {
	// Only add commands here, NOT aliases.
	if (name.equalsIgnoreCase("help")) {
	    cmd_help(parameters, target);
	} else if (name.equalsIgnoreCase("quote")) {
	    cmd_quote(parameters, target);
	} else if (name.equalsIgnoreCase("join")) {
	    cmd_join(parameters, target);
	} else if (name.equalsIgnoreCase("part")) {
	    cmd_part(parameters, target);
	} else if (name.equalsIgnoreCase("ping")) {
	    cmd_ping(parameters, target);
	} else if (name.equalsIgnoreCase("quit")) {
	    cmd_quit(parameters, target);
	} else if (name.equalsIgnoreCase("nick")) {
	    cmd_nick(parameters, target);
	} else if (name.equalsIgnoreCase("me")) {
	    cmd_me(parameters, target);
	} else if (name.equalsIgnoreCase("query")) {
	    cmd_query(parameters, target);
	} else if (name.equalsIgnoreCase("msg")) {
	    cmd_msg(parameters, target);
	} else if (name.equalsIgnoreCase("eirc")) {
	    cmd_eirc(parameters, target);
	} else if (name.equalsIgnoreCase("kban")) {
	    cmd_kban(parameters, target);
	} else if (name.equalsIgnoreCase("ignore")) {
	    cmd_ignore(parameters, target);
	} else if (name.equalsIgnoreCase("unignore")) {
	    cmd_unignore(parameters, target);
	} else if (name.equalsIgnoreCase("server")) {
	    cmd_server(parameters, target);
	}
    }

    void cmd_help(Vector params, OutputWindow target) {
	String help = "";

	// If no command has been specified.
	if (0 == params.size()) {
	    // List internal commands.
	    for (int i = 0; null != user_commands[i]; i++) {
		help = help.concat(PRE_INFO).concat(user_commands[i].getName()).concat("\n");
	    }
	} else {
	    String action = (String) params.elementAt(0);

	    // If it's a local command, resolve aliases.
	    Command c = findCommand(action);
	    if (null != c) {
		action = c.getCommand();
	    }

	    // See if there's help locally provided for this command.
	    try {
		help = getString("help.".concat(action).concat(".short")).concat("\n")
		    .concat(getString("help.".concat(action).concat(".long")));
	    } catch (MissingResourceException e) {
		// Command is unknown.
		Object a[] = { action };
		MessageFormat mf = new MessageFormat(getString("eirc.s21"));
		help = PRE_ERROR.concat(mf.format(a));
	    }

	    help = help.concat("\n");
	}

	target.output(help.concat("\n\n"));
    }

    void cmd_quote(Vector params, OutputWindow target) {
	String a[] = new String [0];
	sendMessage((String) params.elementAt(0), a);
    }

    void cmd_nick(Vector params, OutputWindow target) {
	String a[] = new String [params.size()];
	params.copyInto(a);
	a[0] = RFC1459.filterString(a[0]);

	sendMessage("nick", a);
    }

    void cmd_me(Vector params, OutputWindow target) {
	if (!(target instanceof ChatWindow)) {
	    status.printError(getString("eirc.s23"));
	    return;
	}

	String to = ((ChatWindow) target).getPanelName();
	params.insertElementAt(to, 0);

	String a[] = new String [params.size()];
	params.copyInto(a);

	((ChatWindow) target).printMyAction(a[1]);

	a[1] = "\001ACTION " + a[1] + "\001";
	sendMessage("privmsg", a);
    }

    void cmd_query(Vector params, OutputWindow target) {
	String a[] = new String [params.size()];
	params.copyInto(a);

	openPrivate(a[0]);

	if (2 == params.size()) {
	    sendMessage("privmsg", a);
	    openPrivate(a[0]).printMyPrivmsg(a[1]);
	}
    }

    void cmd_join(Vector params, OutputWindow target) {
	String a[] = new String [params.size()];
	params.copyInto(a);

	if (!Channel.isChannel(a[0])) {
	    a[0] = '#' + a[0];
	}
	sendMessage("join", a);
    }

    void cmd_part(Vector params, OutputWindow target) {
	if (0 == params.size()) {
	    if (!(target instanceof ChannelWindow)) {
		target.printError(getString("eirc.s24"));
		return;
	    }
	    params.insertElementAt(((ChannelWindow) target).getPanelName(), 0);
	}

	String a[] = new String [params.size()];
	params.copyInto(a);

	if (!Channel.isChannel(a[0])) {
	    a[0] = '#' + a[0];
	}

	getChannel(a[0]).dispose();

	// The Listener for the event takes care of sending the message.
//  	sendMessage("part", a);
    }

    void cmd_ping(Vector params, OutputWindow target) {
	params.addElement("\001PING " + (new Date()).getTime() + "\001");

	String a[] = new String [params.size()];
	params.copyInto(a);

	sendMessage("privmsg", a);
    }

    void cmd_quit(Vector params, OutputWindow target) {
	String a[] = new String [Math.max(params.size(), 1)];

	if (0 == params.size()) {
	    a[0] = getConfString("quit_message");
	} else {
	    params.copyInto(a);
	}

	sendMessage("quit", a);
    }

    void cmd_msg(Vector params, OutputWindow target) {
	String a[] = new String [params.size()];
	params.copyInto(a);

	sendMessage("privmsg", a);
    }

    void cmd_eirc(Vector params, OutputWindow target) {
	Object a[] = { PACKAGE, VERSION, getString("author"), getString("update") };
	MessageFormat mf = new MessageFormat(getString("info"));

	target.output(mf.format(a).concat("\n"));
    }

    void cmd_kban(Vector params, OutputWindow target) {
	if (1 == params.size()) {
	    if (!(target instanceof ChannelWindow)) {
		target.printError(getString("eirc.s24"));
		return;
	    }
	    params.insertElementAt(((ChannelWindow) target).getPanelName(), 0);
	}

	String channel = (String) params.elementAt(0);
	if (!Channel.isChannel(channel)) {
	    channel = '#' + channel;
	}

	String nick = (String) params.elementAt(1);

  	sendCommand("mode " + channel + " +b " + nick, target);
  	sendCommand("kick " + channel + " " + nick, target);
    }

    void cmd_ignore(Vector params, OutputWindow target) {
	String nick = (String) params.elementAt(0);

	ignore_list.put(nick, "");

	Object a[] = { nick };
	MessageFormat mf = new MessageFormat(getString("eirc.ignore"));
	target.printError(mf.format(a));
    }

    void cmd_unignore(Vector params, OutputWindow target) {
	String nick = (String) params.elementAt(0);

	ignore_list.remove(nick);

	Object a[] = { nick };
	MessageFormat mf = new MessageFormat(getString("eirc.unignore"));
	target.printError(mf.format(a));
    }

    void cmd_server(Vector params, OutputWindow target) {
	target.printError(getString("eirc.command_notapplicable"));
    }

    public void sendMessage(String command, String parameters[]) {
	if (connected) {
	    LocalMessage m = new LocalMessage(command, parameters);
	    if (debug_traffic) {
		String t = m.toString();
		t = t.substring(0, t.length() - 2);
		System.err.println("> ".concat(t));
	    }
	    try {
		ct.enqueueMessage(m);
	    } catch (IOException e) {
		// This causes troubles.
//    		disconnect();
	    }
	} else {
	    // FIXME: it doesn't distinguish here between user and program generated messages.
//  	    status.printError(getString("eirc.s25"));
	}
    }

    public String getString(String item) {
	return resources.getString(item);
    }

    public String getNick() {
	return current_nick;
    }

    public String getConfString(String opt) {
	return config.getString(opt);
    }

    public boolean getConfBoolean(String opt) {
	return config.getBoolean(opt);
    }

    public int getConfInt(String opt) {
	return config.getInt(opt);
    }

    public void setConfString(String opt, String val) {
	config.setString(opt, val);
    }

    public void setConfBoolean(String opt, boolean state) {
	config.setBoolean(opt, state);
    }

    public void setConfInt(String opt, int val) {
	config.setInt(opt, val);
    }

    public Image getImage(String resource_name) {
	Image image = null;
	String location = images.getString(resource_name);

	try {
	    image = Toolkit.getDefaultToolkit().createImage(Resource.getResource(location, this));
	} catch (IOException e) {
	    System.err.println(e);
	}

	return image;
    }

    public ChannelWindow getChannel(String target) {
	return (ChannelWindow) channels.get(target);
    }

    public ChannelWindow openChannel(String target) {
	ChannelWindow cw = getChannel(target);
	if (null != cw) {
	    return cw;
	}

	cw = new ChannelWindow(this, target, target);
	cw.setFont(getFont());
	cw.setForeground(mainfg);
	cw.setBackground(mainbg);
	cw.setTextForeground(textfg);
	cw.setTextBackground(textbg);

	channels.put(target, cw);
	chat_panel.addChat(cw, target);

  	cw.validate();
	chat_panel.show(target);
  	cw.requestFocus();

	cw.addChatPanelListener(this);

	return cw;
    }

    public PrivateWindow getPrivate(String target) {
	return (PrivateWindow) privates.get(target);
    }

    public PrivateWindow openPrivate(String target) {
	PrivateWindow pw = getPrivate(target);
	if (null != pw) {
	    return pw;
	}

	pw = new PrivateWindow(this, target, target);
	pw.setFont(getFont());
	pw.setForeground(mainfg);
	pw.setBackground(mainbg);
	pw.setTextForeground(textfg);
	pw.setTextBackground(textbg);

	privates.put(target, pw);
	chat_panel.addChat(pw, target);

	pw.validate();
	chat_panel.show(target);
  	pw.requestFocus();

	pw.addChatPanelListener(this);

	return pw;
    }

    public void showPanel(String name) {
	chat_panel.show(name);
    }

    public void openChannelList() {
	if (null != channel_list) {
	    channel_list.loadList(chan_list);
	    channel_list.pack();
//  	    channel_list.setState(Frame.NORMAL);
	    channel_list.toFront();
	    return;
	}

	channel_list = new ChannelList(this, chan_list);
	channel_list.addWindowListener(new WindowAdapter() {
		public void windowClosed(WindowEvent e) {
		    channel_list = null;
		}
	    });

	channel_list.setTitle(makeWindowTitle(channel_list.getTitle()));
	channel_list.setFont(getFont());
	channel_list.setForeground(mainfg);
	channel_list.setBackground(mainbg);
	channel_list.pack();
	channel_list.setResizable(false);
	channel_list.setVisible(true);
    }

    public void closeChannelList() {
	if (null != channel_list) {
	    channel_list.dispose();
	}
    }

    public void openChangeFont() {
	if (null != change_font) {
//  	    change_font.setState(Frame.NORMAL);
	    change_font.toFront();
	    return;
	}

	change_font = new ChangeFont(this, this);
	change_font.addWindowListener(new WindowAdapter() {
		public void windowClosed(WindowEvent e) {
		    change_font = null;
		}
	    });

	change_font.setTitle(makeWindowTitle(change_font.getTitle()));
	change_font.setFont(getFont());
	change_font.setForeground(mainfg);
	change_font.setBackground(mainbg);
	change_font.setTextForeground(textfg);
	change_font.setTextBackground(textbg);
	change_font.pack();
	change_font.setResizable(false);
	change_font.setVisible(true);
    }

    public void closeChangeFont() {
	if (null != change_font) {
	    change_font.dispose();
	}
    }

    public void openNickServ() {
	if (null != nickserv_commander) {
//  	    nickserv_commander.setState(Frame.NORMAL);
	    nickserv_commander.toFront();
	    return;
	}

	nickserv_commander = new NickServCommander(this, getConfString("services_password"), getConfString("services_email"));
	nickserv_commander.addWindowListener(new WindowAdapter() {
		public void windowClosed(WindowEvent e) {
		    nickserv_commander = null;
		}
	    });

	nickserv_commander.setTitle(makeWindowTitle(nickserv_commander.getTitle()));
	nickserv_commander.setFont(getFont());
	nickserv_commander.setBackground(mainbg);
	nickserv_commander.setForeground(mainfg);
	nickserv_commander.setTextBackground(textbg);
	nickserv_commander.setTextForeground(textfg);
	nickserv_commander.pack();
	nickserv_commander.setResizable(false);
	nickserv_commander.setVisible(true);
	nickserv_commander.requestFocus();
    }

    public void openConfigurator() {
	if (null != configurator) {
//  	    configurator.setState(Frame.NORMAL);
	    configurator.toFront();
	    return;
	}

	configurator = new Configurator(this, default_options);
	configurator.addWindowListener(new WindowAdapter() {
		public void windowClosed(WindowEvent e) {
		    configurator = null;
		}
	    });

	configurator.setTitle(makeWindowTitle(configurator.getTitle()));
	configurator.setFont(getFont());
	configurator.setBackground(mainbg);
	configurator.setForeground(mainfg);
	configurator.pack();
	configurator.setResizable(false);
	configurator.setVisible(true);
	configurator.requestFocus();
    }

    public void openNewChannel() {
	if (null != new_channel) {
//  	    new_channel.setState(Frame.NORMAL);
	    new_channel.toFront();
	    return;
	}

	new_channel = new NewChannel(this);
	new_channel.addWindowListener(new WindowAdapter() {
		public void windowClosed(WindowEvent e) {
		    new_channel = null;
		}
	    });

	new_channel.setTitle(makeWindowTitle(new_channel.getTitle()));
	new_channel.setFont(getFont());
	new_channel.setForeground(mainfg);
	new_channel.setBackground(mainbg);
	new_channel.setTextForeground(textfg);
	new_channel.setTextBackground(textbg);
	new_channel.pack();
	new_channel.setResizable(false);
	new_channel.setVisible(true);
	new_channel.requestFocus();
    }

    String makeWindowTitle(String t) {
	String te;
	if (null != (te = getParameter("titleExtra"))) {
	    // Force a space char at the beginning, browsers trim PARAMs.
	    if (' ' != te.charAt(0)) {
		te = ' ' + te;
	    }
	    t = t.concat(te);
	}

	return t;
    }

    public void requestFocus() {
	if (connected) {
	    status.requestFocus();
	} else {
	    control_panel.requestFocus();
	}
    }

    public void setFont(Font f) {
	super.setFont(f);

	// This shouldn't be needed, but it is, or else the Nick entry will grow larger than it should.
	control_panel.validate();

	// Propagate changes to other windows.
	for (Enumeration e = channels.elements(); e.hasMoreElements(); ) {
	    ChannelWindow cw = (ChannelWindow) e.nextElement();
	    cw.setFont(f);
	    cw.validate();
	}
	for (Enumeration e = privates.elements(); e.hasMoreElements(); ) {
	    PrivateWindow pw = (PrivateWindow) e.nextElement();
	    pw.setFont(f);
	    pw.validate();
	}
    }

    public void setBackground(Color c) {
	super.setBackground(c);
	control_panel.setBackground(mainbg);
	chat_panel.setBackground(mainbg);
    }

    public void setForeground(Color c) {
	super.setForeground(c);
	control_panel.setForeground(mainfg);
	chat_panel.setForeground(mainfg);
    }

    public void setTextBackground(Color c) {
  	control_panel.setTextBackground(c);
    }

    public void setTextForeground(Color c) {
  	control_panel.setTextForeground(c);
    }

    public void visitURL(URL url) {
	visitURL(url, "_blank");
    }

    public void visitURL(URL url, String target) {
	getAppletContext().showDocument(url, target);
    }

    public void mouseClicked(MouseEvent ev) {
	Component comp = ev.getComponent();
	String comp_name = comp.getName();

	if (comp_name.equals("connect")) {
	    if (connected) {
		disconnect();
	    } else {
		if (0 == control_panel.getNick().length()) {
		    status.printError(getString("eirc.s1"));
		    return;
		}

		try {
		    connect();
		} catch (IOException e) {
		    System.err.println(e);
		}
	    }
	} else if (comp_name.equals("channel-list")) {
	    String p[] = { getConfString("list_parameter") };
	    sendMessage("list", p);
	    status.printInfo(getString("eirc.s34"));
	} else if (comp_name.equals("new-channel")) {
	    openNewChannel();
	} else if (comp_name.equals("configuration")) {
	    openConfigurator();
	} else if (comp_name.equals("change-font")) {
	    openChangeFont();
	} else if (comp_name.equals("nickserv")) {
	    openNickServ();
	}
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void actionPerformed(ActionEvent ev) {
	Component source = (Component) ev.getSource();
	String comp_name = source.getName();

	if (comp_name.equals("nick-entry")) {
	    new_nick = control_panel.getNick();

	    if (0 == new_nick.length()){
		return;
	    }
	    if (!connected) {
		try {
		    // NICK is set from here, too
		    connect();
		} catch (IOException e) {
		    System.err.println(e);
		}
		return;
	    }
	    String p[] = { new_nick };
	    sendMessage("nick", p);
	}
    }

    public void chatPanelClosing(ChatPanelEvent ev) {
	ChatPanel source = ev.getChatPanel();
	String name = source.getPanelName();

	chat_panel.removeChat(name);
	if (source instanceof PrivateWindow) {
	    privates.remove(name);
	} else if (source instanceof ChannelWindow) {
	    String p[] = { name };
	    sendMessage("part", p);
	}
    }
}

class Command {
    String command;
    int required_params;
    int accepted_params;
    String alias;
    boolean has_alias;

    // Init as an alias to another command.
    Command(String c, Command cmd) {
	command = c;
	required_params = cmd.required_params;
	accepted_params = cmd.accepted_params;
	alias = cmd.command;
	has_alias = true;
    }

    Command(String c, int rp, int ap) {
	command = c;
	required_params = rp;
	accepted_params = ap;
	has_alias = false;
    }

    public int getHashCode() {
	return command.hashCode();
    }

    public int getRequiredParameters() {
	return required_params;
    }

    public int getAcceptedParameters() {
	return accepted_params;
    }

    public String getName() {
	return command;
    }

    public String getCommand() {
	return (has_alias ? alias : command);
    }
}
