/*
Eteria IRC Client, an RFC 1459 compliant client program written in Java.
Copyright (C) 2000  Javier Kohen <jkohen at tough.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.*;
import java.awt.event.*;

public class ImageButton extends Canvas implements MouseListener {
    public final static int ENTERED = 0;
    public final static int EXITED = 1;
    public final static int PRESSED = 2;
    public final static int DISABLED = 3;
    public final static int STATES = 4;

    private MediaTracker tracker;
    private Image stateImages[];
    private boolean clear_background;

    private int state = EXITED;

    private boolean mouseOver = false;
    private boolean mousePressed = false;

    ImageButton() {
	super();

	tracker = new MediaTracker(this);
	stateImages = new Image[STATES];

	addMouseListener(this);
    }

    public void setImage(int st, Image i) {
	tracker.addImage(i, 0);
	stateImages[st] = i;
    }

    public void paint(Graphics g) {
	if (!isEnabled()) {
	    state = DISABLED;
	}
	if (null != stateImages[state]) {
	    g.drawImage(stateImages[state], 0, 0, this);
	}
    }

    public void mouseClicked(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
	mouseOver = true;
	if (PRESSED == state) {
	    return;
	}
	state = mousePressed ? PRESSED : ENTERED;
	clear_background = true;
	repaint();
    }

    public void mouseExited(MouseEvent e) {
	mouseOver = false;
	state = EXITED;
	clear_background = true;
	repaint();
    }

    public void mousePressed(MouseEvent e) {
	mousePressed = true;
	state = PRESSED;
	clear_background = true;
  	repaint();
    }

    public void mouseReleased(MouseEvent e) {
	mousePressed = false;
	state = mouseOver ? ENTERED : EXITED;
	clear_background = true;
	repaint();
    }

    public Dimension getPreferredSize() {
	int w = -1, h = -1;

	try {
	    tracker.waitForID(0);
	} catch (InterruptedException e) {
	    System.err.println(e);
	    return new Dimension(-1, -1);
	}

	for (int i = 0; i < STATES; i++) {
	    if (null != stateImages[i]) {
		w = stateImages[i].getWidth(this);
		h = stateImages[i].getHeight(this);
		break;
	    }
	}

	return new Dimension(w, h);
    }

    public Dimension getMinimumSize() {
	return getPreferredSize();
    }

    public void setEnabled(boolean active) {
	if (isEnabled() != active) {
	    state = EXITED;
	    super.setEnabled(active);
    	    repaint();
	}
    }
}
