/*
Eteria IRC Client, an RFC 1459 compliant client program written in Java.
Copyright (C) 2000  Javier Kohen <jkohen at tough.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.*;
import java.awt.event.*;
import java.text.Collator;
import java.text.CollationKey;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

public class NickList extends Panel implements MouseListener, AdjustmentListener, ItemSelectable {
    public static final int BAN = 0;
    public static final int OP = 1;
    public static final int HALFOP = 2;
    public static final int VOICE = 3;
    public static final int NORMAL = 4;
    public static final int MODES = 5;

    // This char can be any that can't be used as part of a nick.
    private static final char PRE_TITLE = '#';

    private static Collator collator = null;

    private MouseListener mouseListener;
    private ActionListener actionListener;
    private ItemListener itemListener;

    private final Hashtable list[] = new Hashtable[MODES];
    private final String title[] = new String[MODES];
    Vector nicks;
    String last_clicked_nick;

    private MyList screen_list;
    private Scrollbar h_bar;
    private Scrollbar v_bar;

    int list_x;
    int list_y;

    public NickList(String titles[]) {
	this(titles, MODES);
    }

    public NickList(String titles[], int rows) {
	this(titles, rows, false);
    }

    public NickList(String titles[], int rows, boolean multipleMode) {
	this.nicks = new Vector(rows);

	if (null == collator) {
	    collator = RFC1459.getCollator();
	}

	for (int i = 0; i < MODES; i++) {
	    list[i] = new Hashtable(rows);
	    title[i] = titles[i];
	}

	list_x = list_y = 0;

	screen_list = new MyList();
	h_bar = new Scrollbar(Scrollbar.HORIZONTAL);
	v_bar = new Scrollbar(Scrollbar.VERTICAL);

	GridBagLayout gb = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();
	setLayout(gb);

	gbc.fill = GridBagConstraints.BOTH;
	gbc.weightx = 1.0;
	gbc.weighty = 1.0;
	gb.setConstraints(screen_list, gbc);
	add(screen_list);
	gbc.fill = GridBagConstraints.NONE;
	gbc.weightx = 0.0;
	gbc.weighty = 0.0;

	gbc.fill = GridBagConstraints.VERTICAL;
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gb.setConstraints(v_bar, gbc);
	add(v_bar);
	gbc.fill = GridBagConstraints.NONE;
	gbc.gridwidth = 1;

	gbc.fill = GridBagConstraints.HORIZONTAL;
	gb.setConstraints(h_bar, gbc);
	add(h_bar);
	gbc.fill = GridBagConstraints.NONE;

	screen_list.addMouseListener(this);
	h_bar.addAdjustmentListener(this);
	v_bar.addAdjustmentListener(this);

	recalcScrollbar();
    }

    /**
     * Gets the NickItem placed at <code>p</code>.
     *
     * @param p the point in the NickList.
     */
    private NickItem getNickAt(Point p) {
	NickItem[] items;
	synchronized (nicks) {
	    items = new NickItem[nicks.size()];
	    nicks.copyInto(items);
	}

	int i;
	int h = -list_y;
	for (i = 0; i < items.length; i++) {
	    /* FIXME: this code should work, the rest needs modification for this to happen. (The point needs to be translated by list_y before.) */
//  	    Rectangle r = items[i].getBounds();
//  	    if (r.contains(p)) {
//  		break;
//  	    }
	    Dimension d = items[i].getPreferredSize();

	    h += d.height;

	    if (h > p.y) {
		break;
	    }
	}

	return (i == items.length ? null : items[i]);
    }

    // This is only valid for nicks returned by ActionEvent.getActionCommand().
    public static boolean isTitle(String item) {
  	return (PRE_TITLE == item.charAt(0));
    }

    /**
     * Returns a canonical representation of <code>nick</code>, that is, without the mode character.
     *
     * @param nick the nick name to canonicalize.
     */
    static public String canonicalize(String nick) {
	char ch = nick.charAt(0);

	switch (ch) {
	case '@':
	case '%':
	case '+':
	    return nick.substring(1);
	default:
	    return nick;
	}
    }

    static public int charToMode(char mode) {
	switch (mode) {
	case '@':
	    return OP;
	case '%':
	    return HALFOP;
	case '+':
	    return VOICE;
	default:
	    return NORMAL;
	}
    }

    static public int modeCharToMode(char mode) {
	switch (mode) {
	case 'o':
	    return OP;
	case 'h':
	    return HALFOP;
	case 'v':
	    return VOICE;
	case 'b':
	    return BAN;
	default:
	    return NORMAL;
	}
    }

    public NickInfo getNickInfo(String nick) {
	return getNick(nick);
    }

    public void setNickInfo(String nick, NickInfo info) {
	NickItem ni = getNick(nick);
	if (null == ni) {
	    return;
	}

	ni.setInfo(info);

	recalcScrollbar();
    }

    public NickInfo getNickInfoAt(Point p) {
	return (NickInfo) getNickAt(p);
    }

    public synchronized void setBounds(int x, int y, int width, int height) {
        super.setBounds(x, y, width, height);

	recalcScrollbar();
    }

    private synchronized void recalcScrollbar() {
	Rectangle bnd = getBounds();
	if (0 == bnd.width || 0 == bnd.height) {
            h_bar.setEnabled(false);
            v_bar.setEnabled(false);
	    return;
	}

	NickItem[] items;
	synchronized (nicks) {
	    items = new NickItem[nicks.size()];
	    nicks.copyInto(items);
	}

	int list_width = 0;
	int list_height = 0;

	for (int i = 0; i < items.length; i++) {
	    Dimension d = items[i].getPreferredSize();

	    list_width = Math.max(list_width, d.width);
	    list_height += d.height;
	}

        if (bnd.width > list_width + v_bar.getBounds().width) {
            h_bar.setEnabled(false);
        } else {
            h_bar.setEnabled(true);

            int barWidth = v_bar.getPreferredSize().width;
            int vis = bnd.width - barWidth;
            int max = list_width;
            int oldMax = h_bar.getMaximum();

            if (oldMax == 0) {
                list_y = 0;
            } else {
		list_y = (int) (((float) list_y / (float) oldMax) * (float) max);
            }

            h_bar.setValues(list_x, vis, 0, max);
            h_bar.setBlockIncrement(vis / 2);
            h_bar.setUnitIncrement(vis / 4);
	}

	int others_height = h_bar.getPreferredSize().height;

        if (bnd.height > list_height + others_height) {
            v_bar.setEnabled(false);
	    list_y = 0;
        } else {
            v_bar.setEnabled(true);

	    int vis = bnd.height - others_height;
	    int max = list_height;
            int oldMax = v_bar.getMaximum();

            if (oldMax == 0) {
                list_x = 0;
            } else {
		list_x = (int) (((float) list_x / (float) oldMax) * (float) max);
            }

            v_bar.setValues(list_y, vis, 0, max);
            v_bar.setBlockIncrement(vis / 2);
            v_bar.setUnitIncrement(vis / 4);
	}

	screen_list.repaint();
    }

    public void adjustmentValueChanged(AdjustmentEvent e) {
        list_x = h_bar.getValue();
        list_y = v_bar.getValue();
        screen_list.repaint();
    }

    public boolean isBanned(String nick) {
	return list[BAN].containsKey(nick);
    }

    public boolean isOp(String nick) {
	return list[OP].containsKey(nick);
    }

    public boolean isHalfOp(String nick) {
	return list[HALFOP].containsKey(nick);
    }

    public boolean isVoiced(String nick) {
	return list[VOICE].containsKey(nick);
    }

    public boolean isNormal(String nick) {
	return true;
    }

    // Add an un-canonicalized nick
    public void add(String item) {
	int mode = charToMode(item.charAt(0));
	item = NickList.canonicalize(item);

	// Always add the nick to NORMAL list, so if the user is removed from MODE list, it doesn't disappear.
	add(item, NORMAL);
	if (NORMAL != mode && !list[mode].containsKey(item)) {
	    add(item, mode);
	}
    }

    public void add(String item, int mode) {
	NickItem n = getNick(item);
	if (null == n) {
	    n = new NickItem(item);
	}

	list[mode].put(item, n);

	rebuildList();

	n.setVisible(true);

	recalcScrollbar();
    }

    public void change(String nick, String new_nick) {
	NickItem item = null;

	for (int i = 0; i < MODES; i++) {
	    if (list[i].containsKey(nick)) {
		// FIXME: This should be atomic.
		item = (NickItem) list[i].remove(nick);
		list[i].put(new_nick, item);
	    }
	}

	if (null == item) {
	    return;
	}

	item.setNick(new_nick);

	rebuildList();

	recalcScrollbar();
    }

    // Note: binary matching of the nick is expected.
    public void remove(String item) {
	for (int i = 0; i < MODES; i++) {
	    NickItem old_item = (NickItem) list[i].remove(item);
	    if (null != old_item) {
		old_item.setVisible(false);
	    }
	}

	rebuildList();

	recalcScrollbar();
    }

    // Note: binary matching of the nick is expected.
    public void remove(String item, int mode) {
	NickItem old_item = (NickItem) list[mode].remove(item);

	rebuildList();

	recalcScrollbar();
    }

    public void removeAll() {
	for (int i = 0; i < MODES; i++) {
	    list[i].clear();
	}

	rebuildList();

	screen_list.repaint();

	recalcScrollbar();
    }

    private synchronized void rebuildList() {
	synchronized (nicks) {
	    Hashtable listed = new Hashtable();
	    Vector temp_list = new Vector(nicks.size() + 2);
	    int base_index = 0;
	    int list_size = 0;

	    // Build the list off-screen.
	    for (int i = 0; i < MODES; i++) {
		// Don't write headers for empty groups
		if (list[i].isEmpty()) {
		    continue;
		}

		boolean title_added = false;
		for (Enumeration e = list[i].elements(); e.hasMoreElements(); ) {
		    NickItem n = (NickItem) e.nextElement();
		    String s = n.getNick();

		    // Only show the nick once in the topmost ("most important") sublist it belongs to.
		    if (!listed.containsKey(s)) {
			if (!title_added) {
			    title_added = true;
			    NickItem temp_title = new NickItem(title[i], true);
			    temp_title.setVisible(true);
			    temp_list.addElement(temp_title);
			    list_size++;
			    base_index = list_size;
			}

			// Find where this item should go to keep the list sorted.
			int j = base_index;
			for ( ; j < list_size; j++) {
			    NickInfo t = (NickInfo) temp_list.elementAt(j);
			    if (collator.compare(s, t.getNick()) < 0) {
				break;
			    }
			}

			temp_list.insertElementAt(n, j);
			list_size++;
			listed.put(s, "");
		    }
		}
	    }

	    this.nicks = temp_list;
	}
    }

    private NickItem getNick(String n) {
	for (int i = 0; i < MODES; i++) {
	    if (list[i].containsKey(n)) {
		return (NickItem) list[i].get(n);
	    }
	}

	return null;
    }

    public boolean isTitle(int index) {
	if (index < 0) {
	    return false;
	}

	int items = 0;

	for (int i = 0; i < MODES; i++) {
	    if (items == index) {
		// This is a sub title header
		return true;
	    }

	    int s = list[i].size();

	    if (s > 0) {
		items += s + 1;
	    }
	}

	return false;
    }

    /*
     * This method is deprecated, use getSelectedObjects instead.
     */
    public String getSelectedItem() {
	return last_clicked_nick;
    }

    public Object[] getSelectedObjects() {
	Vector temp_list;
	synchronized (nicks) {
	    temp_list = new Vector(nicks.size());

	    for (Enumeration e = nicks.elements(); e.hasMoreElements(); ) {
		NickItem n = (NickItem) e.nextElement();
		if (n.isSelected()) {
		    temp_list.addElement((NickInfo) n);
		}
	    }
	}

	if (!temp_list.isEmpty()) {
	    NickInfo[] items = new NickInfo[temp_list.size()];
	    temp_list.copyInto(items);

	    return items;
	}

	return null;
    }

    private MouseEvent translateListMouseEvent(MouseEvent ev) {
	// We don't need to translate the position because the screen_list shares the same origin with the NickList.
	return new MouseEvent(this,
			      ev.getID(),
			      ev.getWhen(),
			      ev.getModifiers(),
			      ev.getX(),
			      ev.getY(),
			      ev.getClickCount(),
			      ev.isPopupTrigger());
    }

    public void addMouseListener(MouseListener l) {
	if (null != l) {
	    mouseListener = AWTEventMulticaster.add(mouseListener, l);
	}
    }

    public void removeMouseListener(MouseListener l) {
	if (null != l) {
	    mouseListener = AWTEventMulticaster.remove(mouseListener, l);
	}
    }

    public void addActionListener(ActionListener l) {
	if (null != l) {
	    actionListener = AWTEventMulticaster.add(actionListener, l);
	}
    }

    public void removeActionListener(ActionListener l) {
	if (null != l) {
	    actionListener = AWTEventMulticaster.remove(actionListener, l);
	}
    }

    protected void processActionEvent(ActionEvent e) {
	if (actionListener != null) {
	    actionListener.actionPerformed(e);
	}
    }

    public void addItemListener(ItemListener l) {
	if (null != l) {
	    itemListener = AWTEventMulticaster.add(itemListener, l);
	}
    }

    public void removeItemListener(ItemListener l) {
	if (null != l) {
	    itemListener = AWTEventMulticaster.remove(itemListener, l);
	}
    }

    protected void processItemEvent(ItemEvent e) {
	if (itemListener != null) {
	    itemListener.itemStateChanged(e);
	}
    }

    public void mouseClicked(MouseEvent ev) {
	Object source = ev.getSource();

	if (source.equals(screen_list)) {
	    if (mouseListener != null) {
		mouseListener.mouseClicked(translateListMouseEvent(ev));
	    }

	    NickItem nick = getNickAt(ev.getPoint());
	    if (null == nick) {
		return;
	    }

	    int click_count = ev.getClickCount();
	    if (1 == click_count) {
		last_clicked_nick = nick.getNick();
		nick.setSelected(!nick.isSelected());
	    } else if (2 == click_count) {
		String nick_name = nick.getNick();

		// Make this nick easy to distinguish from a user nick.
		if (nick.isTitle()) {
		    nick_name = PRE_TITLE + nick_name;
		}

		processActionEvent(new ActionEvent(NickList.this,
						   ActionEvent.ACTION_PERFORMED,
						   nick_name,
						   ev.getModifiers()));
	    }
	}
    }

    public void mousePressed(MouseEvent ev) {
	Object source = ev.getSource();

	if (source.equals(screen_list)) {
	    if (mouseListener != null) {
		mouseListener.mousePressed(translateListMouseEvent(ev));
	    }
	}
    }

    public void mouseReleased(MouseEvent ev) {
	Object source = ev.getSource();

	if (source.equals(screen_list)) {
	    if (mouseListener != null) {
		mouseListener.mouseReleased(translateListMouseEvent(ev));
	    }
	}
    }

    public void mouseEntered(MouseEvent ev) {
	Object source = ev.getSource();

	if (source.equals(this)) {
	    if (mouseListener != null) {
		mouseListener.mouseEntered(ev);
	    }
	}
    }

    public void mouseExited(MouseEvent ev) {
	Object source = ev.getSource();

	if (source.equals(this)) {
	    if (mouseListener != null) {
		mouseListener.mouseExited(ev);
	    }
	}
    }

    private class MyList extends Canvas {
	private Image image;
	private Graphics img_gfx;

	private void createImageBuffer() {
	    Dimension size = getSize();

	    image = createImage(size.width, size.height);
	    if (null != img_gfx) {
		img_gfx.dispose();
	    }
	    img_gfx = image.getGraphics();
	}

	public void update(Graphics g) {
	    paint(g);
	}

	public void paint(Graphics g) {
	    Dimension size = getSize();

	    if (null == img_gfx) {
		createImageBuffer();
	    }

	    img_gfx.setColor(getBackground());
	    img_gfx.fillRect(0, 0, size.width, size.height);
	    img_gfx.setColor(getForeground());

	    NickItem[] items;
	    synchronized (nicks) {
		items = new NickItem[nicks.size()];
		nicks.copyInto(items);
	    }

	    Dimension d;

	    int y = 0;
	    for (int i = 0; i < items.length; i++, y += d.height) {
		// Don't paint past the end of the visible area.
		if (y >= list_y + size.height) {
		    break;
		}

		d = items[i].getPreferredSize();

		// Don't paint before the beginning of the visible area.
		if (y <= list_y - d.height) {
		    continue;
		}

		Graphics ng = img_gfx.create(0, y - list_y,
					     size.width, d.height);
		items[i].paint(ng);
	    }

	    g.drawImage(image, 0, 0, this);
	}
    }

    class NickItem extends Canvas implements NickInfo {
	boolean visible;
	boolean title;
	boolean selected;
	boolean dirty_info;
	String nick;

	Dimension size;
	int text_baseline;

	FontMetrics fm;

	NickItem(String nick) {
	    this(nick, false);
	}

	NickItem(String nick, boolean title) {
	    this((NickInfo) new SimpleNickInfo(title, nick));
	}

	NickItem(NickInfo info) {
	    this.visible = false;
	    this.selected = false;

	    setInfo(info);
	}

	public String toString() {
	    return getNick();
	}

	public void paint(Graphics g) {
	    calcSize();

	    Color text_color = g.getColor();
	    int x;
	    Rectangle bounds = g.getClipBounds();

	    if (isTitle()) {
		// Set background color.
		g.setColor(Color.gray);
		g.fillRect(bounds.x, bounds.y, bounds.width, bounds.height);

		x = (g.getClipBounds().width - size.width) / 2;
	    } else if (isSelected()) {
		// Set background color.
		g.setColor(Color.darkGray);
		g.fillRect(bounds.x, bounds.y, bounds.width, bounds.height);

		x = -list_x;
	    } else {
		x = -list_x;
	    }

	    // Restore foreground color.
	    g.setColor(text_color);

	    g.drawString(getNick(), x, text_baseline);
	}

	public Dimension getMinimumSize() {
	    return getPreferredSize();
	}

	public Dimension getPreferredSize() {
//      	    Thread.dumpStack();
	    calcSize();

	    return (isVisible() ? size : new Dimension(0, 0));
	}

//  	public Rectangle getBounds() {
//  	    Rectangle r = super.getBounds();
//  	    System.err.println("DEBUG: getBounds = " + r);
//  	    return r;
//  	}

//  	public void setBounds(int x, int y, int h, int w) {
//  	    super.setBounds(x, y, h, w);
//  	    System.err.println("DEBUG: setBounds = " + x + ";" + y + ";" + h + ";" + w);
//  	}

	public boolean isVisible() {
	    return visible;
	}

	public void setVisible(boolean visible) {
	    this.visible = visible;

	    screen_list.repaint();
	}

	public boolean isTitle() {
	    return title;
	}

	public void setTitle(boolean title) {
	    this.title = title;

	    if (isVisible()) {
		screen_list.repaint();
	    }
	}

	public boolean isSelected() {
	    return selected;
	}

	public void setSelected(boolean selected) {
	    this.selected = selected;

	    int item_state = selected ?
		ItemEvent.SELECTED :
		ItemEvent.DESELECTED;

	    processItemEvent(new ItemEvent(NickList.this,
					   ItemEvent.ITEM_STATE_CHANGED,
					   this,
					   item_state));

	    if (isVisible()) {
		screen_list.repaint();
	    }
	}

	public String getNick() {
	    return nick;
	}

	public void setNick(String nick) {
	    this.nick = nick;
	    this.dirty_info = true;
	}

	public NickInfo getInfo() {
	    return this;
	}

	public void setInfo(NickInfo info) {
	    setTitle(info.isTitle());
	    setNick(info.getNick());

	    this.dirty_info = true;

	    if (isVisible()) {
		screen_list.repaint();
	    }
	}

	private synchronized void calcSize() {
	    Graphics g = NickList.this.getGraphics();
	    if (null == g) {
		fm = null;
		size = new Dimension(0, 0);
		text_baseline = 0;
		return;
	    }

	    FontMetrics new_fm = g.getFontMetrics();
	    if (!dirty_info && new_fm.equals(fm)) {
		return;
	    }
	    fm = new_fm;

	    Dimension text_size = new Dimension(fm.stringWidth(getNick()),
						fm.getLeading() + fm.getMaxAscent() + fm.getMaxDescent());

	    size = text_size;
	    text_baseline = text_size.height - 1;

	    this.dirty_info = true;
	}
    }
}
