/*
Eteria IRC Client, an RFC 1459 compliant client program written in Java.
Copyright (C) 2000  Javier Kohen <jkohen at tough.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.*;
import java.net.MalformedURLException;
import java.net.URL;
import com.lyrisoft.awt.*;

public abstract class OutputWindow extends ChatPanel implements HyperlinkReceiver {
    public static Font DEFAULT_FONT;

    public TextStyle ERROR_STYLE;
    public TextStyle INFO_STYLE;
    public TextStyle WARNING_STYLE;

    public TextStyle NICK_STYLE;
    public TextStyle NOTICE_STYLE;

    protected HyperlinkTextView text_canvas;
    protected ScrollView scroll_text_canvas;

    static {
	DEFAULT_FONT = TextView.DEFAULT_FONT;
    }

    public OutputWindow(String title) {
	super(title);

	Font bold_font = new Font(DEFAULT_FONT.getName(), Font.BOLD, DEFAULT_FONT.getSize());

	ERROR_STYLE = new TextStyle(bold_font, Color.red);
	INFO_STYLE = new TextStyle(bold_font, Color.black);
	WARNING_STYLE = new TextStyle(DEFAULT_FONT, Color.red);

	NICK_STYLE = new TextStyle(bold_font, Color.black);
	NOTICE_STYLE = new TextStyle(DEFAULT_FONT, Color.blue);

	text_canvas = new HyperlinkTextView(true, this);
	text_canvas.setForeground(Color.black);
	scroll_text_canvas = new ScrollView(text_canvas);
  	scroll_text_canvas.setSize(300, 200);
    }

    /* High level methods.
     */
    public void printError(String s) {
	String prefix = getString("eirc.pre.error");

	text_canvas.append(prefix.concat(s).concat("\n"), ERROR_STYLE);

	postTextEvent();
    }

    public void printInfo(String s) {
	String prefix = getString("eirc.pre.info");

	text_canvas.append(prefix.concat(s).concat("\n"), INFO_STYLE);

	postTextEvent();
    }

    public void printWarning(String s) {
	String prefix = getString("eirc.pre.warning");

	text_canvas.append(prefix.concat(s).concat("\n"), WARNING_STYLE);

	postTextEvent();
    }

    public void printNotice(String s, String whom) {
	String prefix = "-".concat(whom).concat("- ");

	text_canvas.append(prefix, NICK_STYLE);
	text_canvas.append(s.concat("\n"), NOTICE_STYLE);

	postTextEvent();
    }

    /* Low level methods.
     */
    public void output(String s) {
	text_canvas.append(s);

	postTextEvent();
    }

    public void setFont(Font f) {
	super.setFont(f);

	Font bold_font = new Font(f.getName(), Font.BOLD, f.getSize());

	DEFAULT_FONT = f;

	ERROR_STYLE.setFont(bold_font);
	INFO_STYLE.setFont(bold_font);
	WARNING_STYLE.setFont(f);

	NICK_STYLE.setFont(bold_font);
	NOTICE_STYLE.setFont(f);

	text_canvas.setFont(f);
    }

    public void handleHyperlink(String link) {
	if (Channel.isChannel(link)) {
	    joinChannel(link);
 	} else {
	    try {
		visitURL(new URL(link));
	    } catch (MalformedURLException e) {
		printError(getString("eirc.not_an_url"));
	    }
	}
    }

    /* Returns a configuration boolean. */
    protected abstract boolean getConfBoolean(String key);

    /* Returns an i18n string. */
    protected abstract String getString(String key);

    /* Makes the client join the specified Channel. */
    protected abstract void joinChannel(String name);

    /* Makes the browser visit the specified URL. */
    protected abstract void visitURL(URL url);
}
