/*
Eteria IRC Client, an RFC 1459 compliant client program written in Java.
Copyright (C) 2000  Javier Kohen <jkohen at tough.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import java.util.Date;

import com.lyrisoft.awt.*;
import htmllayout.HtmlLayout;

class PrivateWindow extends ChatWindow implements History, ActionListener, MouseListener {
    private TextFieldHistory entry;
    private ImageButton whois;
    private ImageButton close;

    // Purge entries past this number.
    private int history_max_size = 200;
    private int rows = 0;

    private ServerProcess sp;
    private String user;
    private boolean is_active = false;

    public PrivateWindow(ServerProcess spr, String u, String title) {
	super(title);

  	this.sp = spr;
	this.user = u;

//  	String html_layout;

//  	try {
//  	    html_layout = new String(Resource.getResource("PrivateWindow.html", this));
//  	} catch (java.io.IOException e) {
//  	    System.err.println(e);
//  	    return;
//  	}

//  	setLayout(new HtmlLayout(html_layout));

	GridBagLayout gb = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();

	setLayout(gb);

	gbc.insets = new Insets(2, 2, 2, 2);
	gbc.gridy = 0;

	/* Component's initialization
	 */

	whois = new ImageButton();
	whois.setImage(ImageButton.ENTERED,
		       sp.getImage("private.whois.entered"));
	whois.setImage(ImageButton.EXITED,
		       sp.getImage("private.whois.exited"));
	whois.setImage(ImageButton.PRESSED,
		       sp.getImage("private.whois.pressed"));
	gb.setConstraints(whois, gbc);
  	add(whois/*, "whois"*/);

	close = new ImageButton();
	close.setImage(ImageButton.ENTERED,
		       sp.getImage("private.close.entered"));
	close.setImage(ImageButton.EXITED,
		       sp.getImage("private.close.exited"));
	close.setImage(ImageButton.PRESSED,
		       sp.getImage("private.close.pressed"));
	gbc.anchor = GridBagConstraints.EAST;
	gb.setConstraints(close, gbc);
  	add(close/*, "close"*/);
	gbc.anchor = GridBagConstraints.CENTER;

	/* Next Row */
	gbc.gridy++;
	gbc.gridwidth = GridBagConstraints.REMAINDER;

	gbc.weightx = 1.0;
	gbc.weighty = 1.0;
	gbc.fill = GridBagConstraints.BOTH;
	gb.setConstraints(scroll_text_canvas, gbc);
	add(scroll_text_canvas/*, "scroll_text_canvas"*/);
	gbc.weighty = 0.0;

	/* Next Row */
	gbc.gridy++;

	entry = new TextFieldHistory(80);
	gbc.weightx = 1.0;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gb.setConstraints(entry, gbc);
	add(entry/*, "entry"*/);

	/* Event Listeners
	 */

  	whois.addMouseListener(this);
  	close.addMouseListener(this);
	entry.addActionListener(this);
    }

    public void requestFocus() {
	entry.requestFocus();
    }

    public void trimHistory(int new_size) {
//  	int end;

//  	while (new_size < rows) {
//  	    String t = text_canvas.getText();

//  	    if (-1 == (end = t.indexOf('\n'))) {
//  		break;
//  	    }

//  	    // Replace extra lines with nothing.
//  	    text_canvas.replaceRange("", 0, end + 1);
//  	    rows--;
//  	}
    }

    public void output(String s) {
	int i = 0;
	// Count rows in input.
	while (-1 != (i = s.indexOf('\n', i + 1))) {
	    rows++;
	}
	trimHistory(history_max_size);
  	text_canvas.append(s);
    }

    protected String getNick() {
	return sp.getNick();
    }

    protected boolean getConfBoolean(String key) {
	return sp.getConfBoolean(key);
    }

    protected String getString(String key) {
	return sp.getString(key);
    }

    protected void visitURL(URL url) {
	sp.visitURL(url);
    }

    protected void joinChannel(String name) {
	ChannelWindow cw = sp.getChannel(name);
	if (null == cw) {
	    String p[] = { name };
	    sp.sendMessage("join", p);
	} else {
	    sp.showPanel(name);
	}
    }

    public int getMaximumHistorySize() {
	return history_max_size;
    }

    public void setMaximumHistorySize(int new_size) {
	history_max_size = new_size; 
	trimHistory(history_max_size);
    }

    public void setTextBackground(Color c) {
	text_canvas.setBackground(c);
	entry.setBackground(c);
    }

    public void setTextForeground(Color c) {
	text_canvas.setForeground(c);
	entry.setForeground(c);
    }

    public void actionPerformed(ActionEvent ev) {
	Object comp = ev.getSource();

	if (comp.equals(entry)) {
	    boolean isCommand = false;

	    String text = entry.getText();
	    if (0 == text.length()) {
		return;
	    }

	    if (text.charAt(0) == '/') {
		if (1 == text.length()) {
		    return;
		}
		isCommand = true;
		text = text.substring(1);
	    }

	    if (isCommand) {
		Component source = (Component) ev.getSource();
		sp.sendCommand(text, (OutputWindow) source.getParent());
	    } else {
		String p[] = { user, entry.getText() };
		sp.sendMessage("privmsg", p);
		printMyPrivmsg(entry.getText());
	    }

	    entry.setText("");
	}
    }

    public void mouseClicked(MouseEvent ev) {
	Component comp = ev.getComponent();

	if (comp.equals(whois)) {
	    String p[] = { user };
	    if (null != p[0]) {
		sp.sendMessage("whois", p);
	    }
	} else if (comp.equals(close)) {
	    dispose();
	}
    }

    public void mousePressed(MouseEvent ev) {
    }

    public void mouseReleased(MouseEvent ev) {
    }

    public void mouseEntered(MouseEvent ev) {
    }

    public void mouseExited(MouseEvent ev) {
    }
}
