/*
Eteria IRC Client, an RFC 1459 compliant client program written in Java.
Copyright (C) 2000  Javier Kohen <jkohen at tough.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.*;
import java.awt.event.*;
import java.net.URL;

public class StatusWindow extends OutputWindow implements History, ActionListener {
    private ServerProcess sp;
    private String title;

    // Purge entries past this number.
    private int history_max_size = 200;
    private int rows = 0;

    private TextFieldHistory entry;

    public StatusWindow(ServerProcess sp, String title) {
	super(title);

	this.sp = sp;
	this.title = title;

	GridBagLayout gb = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();

	setLayout(gb);

  	gbc.weightx = 1.0;
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gbc.insets = new Insets(2, 2, 2, 2);

	gbc.fill = GridBagConstraints.BOTH;
	gbc.weighty = 1.0;
	gb.setConstraints(scroll_text_canvas, gbc);
	add(scroll_text_canvas);
	gbc.weighty = 0.0;

	gbc.gridy = 1;

	entry = new TextFieldHistory();
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gb.setConstraints(entry, gbc);
	add(entry);

	entry.addActionListener(this);

//  	text_canvas.addTextListener(new TextListener() {
//  		public void textValueChanged(TextEvent e) {
//  		    if (!is_active) {
//  			messageNotify(true);
//  		    }
//  		}
//  	    });
    }

    public void trimHistory(int new_size) {
//  	int end;

//  	while (new_size < rows) {
//  	    String t = text_canvas.getText();

//  	    if (-1 == (end = t.indexOf('\n'))) {
//  		break;
//  	    }

//  	    // Replace extra lines with nothing.
//  	    text_canvas.replaceRange("", 0, end + 1);
//  	    rows--;
//  	}
    }

    public void output(String s) {
	int i = 0;
	// Count rows in input.
	while (-1 != (i = s.indexOf('\n', i + 1))) {
	    rows++;
	}
	trimHistory(history_max_size);
  	text_canvas.append(s);
    }

    public void requestFocus() {
	entry.requestFocus();
    }

    public void setTextBackground(Color c) {
	text_canvas.setBackground(c);
	entry.setBackground(c);
    }

    public void setTextForeground(Color c) {
	text_canvas.setForeground(c);
	entry.setForeground(c);
    }

    protected boolean getConfBoolean(String key) {
	return sp.getConfBoolean(key);
    }

    protected String getString(String key) {
	return sp.getString(key);
    }

    protected void visitURL(URL url) {
	sp.visitURL(url);
    }

    protected void joinChannel(String name) {
	ChannelWindow cw = sp.getChannel(name);
	if (null == cw) {
	    String p[] = { name };
	    sp.sendMessage("join", p);
	} else {
	    sp.showPanel(name);
	}
    }

    public int getMaximumHistorySize() {
	return history_max_size;
    }

    public void setMaximumHistorySize(int new_size) {
	history_max_size = new_size; 
	trimHistory(history_max_size);
    }

    public void actionPerformed(ActionEvent ev) {
	Component source = (Component) ev.getSource();

	if (source.equals(entry)) {
	    String text = entry.getText();
	    if (0 == text.length()) {
		return;
	    }

	    if (text.charAt(0) == '/') {
		if (1 == text.length()) {
		    return;
		}
		text = text.substring(1);
	    }

	    sp.sendCommand(text, this);
	    entry.setText("");
	}
    }
}
