/*
Eteria IRC Client, an RFC 1459 compliant client program written in Java.
Copyright (C) 2000  Javier Kohen <jkohen at tough.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.Enumeration;
import java.util.Vector;

class Table extends Panel implements ItemSelectable {
    ItemListener itemListener;

    int visibleRows;
    int rows, cols;
    int cellHeight;
    Vector cellWidth;
    int lineWidth;
    int ipadx = 5, ipady = 5;
    int max_width = Integer.MAX_VALUE;
    boolean wholeRow = false;
    boolean wholeCol = false;
    boolean selectOne = true;

    Header header_canvas;
    Body body;
    Scrollbar sb;

    Vector header;
    Vector data;

    boolean drawHeader = false;

    boolean isSizeOld = true;

    Table() {
	this(0, 0, 4);
    }

    // If R or C are zero, it's set to auto fill
    Table(int r, int c, int visRows) {
	super();

	GridBagLayout gb = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();

	setLayout(gb);

	rows = r;
	cols = c;
	visibleRows = visRows;

	data = new Vector(rows * cols, cols == 0 ? rows : cols);
	header = new Vector(cols);
	cellWidth = new Vector(cols);

	header_canvas = new Header();
	gb.setConstraints(header_canvas, gbc);
	add(header_canvas);

	gbc.gridy = 1;

	body = new Body();
	body.addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent e) {
		    if (1 == e.getClickCount()) {
			selectCell(getCellFromCoord(e.getPoint()));
		    }
		}
	    });
	gbc.weightx = gbc.weighty = 1.0;
	gbc.fill = GridBagConstraints.BOTH;
	gb.setConstraints(body, gbc);
	gbc.weightx = gbc.weighty = 1.0;
	gbc.fill = GridBagConstraints.NONE;
	add(body);

  	sb = new Scrollbar(Scrollbar.VERTICAL, 0, visibleRows, 0, visibleRows);
	sb.setEnabled(false);
	sb.addAdjustmentListener(new AdjustmentListener() {
		public void adjustmentValueChanged(AdjustmentEvent e) {
		    body.setVisible(e.getValue());
		}
	    });
	gbc.fill = GridBagConstraints.VERTICAL;
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gb.setConstraints(sb, gbc);
	gbc.fill = GridBagConstraints.NONE;
	gbc.gridwidth = 1;
	add(sb);
    }

    public void addMouseListener(MouseListener l) {
	header_canvas.addMouseListener(l);
	body.addMouseListener(l);
    }

    public void setFont(Font t) {
	super.setFont(t);
	if (!getFont().equals(t)) {
	    isSizeOld = true;
	}
    }

    public void setHeader(String h[]) {
	for (int i = 0; i < h.length; i++) {
	    header.addElement(h[i]);
	}
	enableHeader(true);
    }

    public void enableHeader(boolean active) {
	if (drawHeader != active) {
	    drawHeader = active;
	    isSizeOld = true;
	}
    }

    public int getIPadX() {
	return (ipadx);
    }

    public void setIPadX(int ix) {
	ipadx = ix;
    }

    public int getIPadY() {
	return (ipady);
    }

    public void setIPadY(int iy) {
	ipady = iy;
    }

    public String getCell(Point p) {
	Dimension f = body.getEfectiveFormat();
	int c = f.width;

	Cell el = (Cell) data.elementAt(p.x + p.y * c);

	return el.toString();
    }

    public Vector getRow(int row) {
	Dimension f = body.getEfectiveFormat();
	int c = f.width;

	Vector v = new Vector(c);

	for (int i = row * c; i < (row + 1) * c; i++) {
	    Cell el = (Cell) data.elementAt(i);
	    v.addElement(el.toString());
	}

	return (v);
    }

    public Vector getCol(int col) {
	Dimension f = body.getEfectiveFormat();
	int r = f.height, c = f.width;

	Vector v = new Vector(r);

	for (int i = col; i < r * c; i += c) {
	    Cell el = (Cell) data.elementAt(i);
	    v.addElement(el.toString());
	}

	return (v);
    }

    public int getRows() {
	return (rows);
    }

    public void setRows(int r) {
	rows = r;
    }

    public int getColumns() {
	return (cols);
    }

    public void setColumns(int c) {
	cols = c;
    }

    public void add(String s) {
	Dimension f = body.getEfectiveFormat();

	data.addElement(new Cell(s));
	isSizeOld = true;
	invalidate();

	int max = data.size() / f.width;
	if (max > visibleRows) {
	    sb.setEnabled(true);
	    sb.setMaximum(max);
	} else {
	    sb.setEnabled(false);
	}
    }

    public void add(int pos, String s) {
	Dimension f = body.getEfectiveFormat();

	if (pos > data.size()) {
	    pos = data.size();
	}

	data.insertElementAt(new Cell(s), pos);
	isSizeOld = true;
	invalidate();

	int max = data.size() / f.width;
	if (max > visibleRows) {
	    sb.setEnabled(true);
	    sb.setMaximum(max);
	} else {
	    sb.setEnabled(false);
	}
    }

    public void remove(int pos) {
	if (pos >= data.size()) {
	    throw new ArrayIndexOutOfBoundsException();
	}

	data.removeElementAt(pos);
	isSizeOld = true;
	invalidate();

	Dimension f = body.getEfectiveFormat();

	int max = data.size() / f.width;
	if (max > visibleRows) {
	    sb.setEnabled(true);
	    sb.setMaximum(max);
	} else {
	    sb.setEnabled(false);
	}
    }

    public void removeAll() {
	data.removeAllElements();
	isSizeOld = true;
	invalidate();
	sb.setEnabled(false);
	body.repaint();
    }

    public int getMaxWidth() {
	return max_width;
    }

    public void setMaxWidth(int w) {
	max_width = w;
    }

    private synchronized void doCellSize() {
	if (!isSizeOld) {
	    return;
	}

	isSizeOld = false;

	int sb_width = sb.getSize().width;
	if (0 == sb_width) {
	    sb_width = 32; // Let's guess here.
	}

	int max_body_width = max_width - sb_width;

  	FontMetrics fm = getGraphics().getFontMetrics();

	Dimension d = body.getEfectiveFormat();
	int r = d.height, c = d.width;

	cellWidth.removeAllElements();
	lineWidth = 0;

	cellHeight = fm.getHeight();

	for (int i = 0; i < c; i++) {
	    int len = 0;
	    int off = i;

	    if (drawHeader) {
		int l = fm.stringWidth((String) header.elementAt(i));
		if (l > len) {
		    len = l;
		}
	    }

	    for (int j = 0; j < r; j++) {
  		int l = fm.stringWidth(((Cell) data.elementAt(off)).toString());
		if (l > len) {
		    len = l;
		}

		off += c;
	    }

	    lineWidth += len + 2 * ipadx;

	    if (0 != max_body_width && lineWidth >= max_body_width) {
		len -= max_body_width - lineWidth;
		lineWidth = max_body_width;
	    }

	    cellWidth.addElement(new Integer(len));
	}
    }

    public Object[] getSelectedObjects() {
	// Let's roughly guess at capacity
	int capacity = 1;
	if (wholeRow) {
	    capacity = cols;
	}
	if (wholeCol) {
	    capacity = rows;
	}

	Vector v = new Vector(capacity);

	for (Enumeration e = data.elements(); e.hasMoreElements(); ) {
	    Cell el = (Cell) e.nextElement();
	    if (el.isSelected()) {
		v.addElement(el.toString());
	    }
	}

	int size = v.size();

	if (0 == size) {
	    return null;
	} else {
	    String[] s = new String [v.size()];
	    v.copyInto(s);
	    return s;
	}
    }

    public void setBackground(Color c) {
	super.setBackground(c);

	header_canvas.setBackground(c);
	body.setBackground(c);
	sb.setBackground(c);
    }

    public void setForeground(Color c) {
	super.setForeground(c);

	header_canvas.setForeground(c);
	body.setForeground(c);
	sb.setForeground(c);
    }

    public void addItemListener(ItemListener l) {
	if (null != l) {
	    itemListener = AWTEventMulticaster.add(itemListener, l);
	}
    }

    public void removeItemListener(ItemListener l) {
	if (null != l) {
	    itemListener = AWTEventMulticaster.remove(itemListener, l);
	}
    }

    protected void processItemEvent(ItemEvent e) {
	if (itemListener != null) {
	    itemListener.itemStateChanged(e);
	}
    }

    public void selectCell(Point p) {
	if (-1 == p.y || -1 == p.x) {
	    return;
	}

	Dimension f = body.getEfectiveFormat();
	int w = f.width;
	int h = f.height;

	Cell el = (Cell) data.elementAt((int) (p.y * w + p.x));
	boolean newState = !el.isSelected();

	if (selectOne) {
	    for (Enumeration e = data.elements(); e.hasMoreElements(); ) {
		((Cell) e.nextElement()).select(false);
	    }
	}

	if (!wholeRow && !wholeCol) {
	    el.select(newState);
	}
	if (wholeRow) {
	    for (int i = 0; i < w; i++) {
		((Cell) data.elementAt((int) (i + p.y * w))).select(newState);
	    }
	}
	if (wholeCol) {
	    for (int i = 0; i < h; i++) {
		((Cell) data.elementAt((int) (i * w + p.x))).select(newState);
	    }
	}

	body.repaint();
    }

    public void selectWholeRow(boolean state) {
	wholeRow = state;
    }

    public void selectWholeCol(boolean state) {
	wholeCol = state;
    }

    public void selectJustOne(boolean state) {
	selectOne = state;
    }

    public Point getCellFromCoord(Point coord) {
	return body.getCellFromCoord(coord);
    }

    class Header extends Canvas {
	final int bottom_line_ipad = 2;

	public void paint(Graphics g) {
	    if (!drawHeader) {
		return;
	    }

	    doCellSize();

	    FontMetrics fm = getGraphics().getFontMetrics();

	    Dimension d = body.getEfectiveFormat();
	    int c = d.width;

	    int x = ipadx;
	    int y = cellHeight + ipady - 1;
	    int w;
	    for (int i = 0; i < c; i++) {
		w = ((Integer) cellWidth.elementAt(i)).intValue();

		String str = (String) header.elementAt(i);

		int l = fm.stringWidth(str);

		// Draw the title centered on the column. 
		g.drawString(str, x + (w - l) / 2, y);
		x += w + 2 * ipadx;
	    }

	    // Draw a separation bar at the bottom of the header.
	    int bottom_y = y + ipady - 2;
	    g.drawLine(bottom_line_ipad - 1, bottom_y, x - ipadx - bottom_line_ipad - 1, bottom_y);
	}

	public Dimension getPreferredSize() {
	    if (!drawHeader) {
		return new Dimension(0, 0);
	    }

	    doCellSize();

	    return new Dimension(lineWidth, cellHeight + 2 * ipady);
	}

	public Dimension getMinimumSize() {
  	    return getPreferredSize();
	}
    }

    class Body extends Canvas {
	int visibleRow = 0;

	Image offScrImage;
	Graphics offScrGfx;

	Body() {
	    super();
	}

	public void paint(Graphics g) {
	    // FIXME: certify the code block below is correct. It does work, though.
	    if (null == offScrImage) {
		update(g);
		return;
	    }

	    Color selbg, selfg, deffg;

	    // SEL[BF]G are logically reversed on purpose.
	    selbg = deffg = getForeground();
	    selfg = getBackground();

	    doCellSize();

	    Dimension d = getEfectiveFormat();
	    int r = d.height, c = d.width;

	    int x = ipadx;
	    int w;
	    for (int i = 0; i < c; i++) {
		int y = cellHeight + ipady - 1;
		int off;

		w = ((Integer) cellWidth.elementAt(i)).intValue();
		off = i + c * visibleRow;
		for (int j = visibleRow; j < Math.min(r, visibleRow + visibleRows); j++) {
		    Cell el = (Cell) data.elementAt(off);
		    if (el.isSelected()) {
			g.setColor(selbg);
			int fillX = x - ipadx;
			int fillY = y - ipady - cellHeight + 1;
			int fillW = w + 2 * ipadx;
			int fillH = cellHeight + 2 * ipady;
			g.fillRect(fillX, fillY, fillW, fillH);
			g.setColor(selfg);
			g.drawString(el.toString(), x, y);
			g.setColor(deffg);
		    } else {
			g.drawString(el.toString(), x, y);
		    }
		    y += cellHeight + 2 * ipady;
		    off += c;
		}

		x += w + 2 * ipadx;
	    }
	}

	public void update(Graphics g) {
	    Dimension size = getSize();

	    // If size has increased, create a new image.
	    if (null == offScrGfx ||
		size.height > offScrImage.getHeight(this) ||
		size.width > offScrImage.getWidth(this)) {
		offScrImage = createImage(size.width, size.height);
		offScrGfx = offScrImage.getGraphics();
	    }

	    offScrGfx.setColor(getBackground());
	    offScrGfx.fillRect(0, 0, size.width, size.height);
	    offScrGfx.setColor(getForeground());

	    paint(offScrGfx);

	    g.drawImage(offScrImage, 0, 0, null);
	}

	public Dimension getPreferredSize() {
	    doCellSize();

	    return new Dimension(lineWidth, visibleRows * (cellHeight + 2 * ipady));
	}

	public Dimension getMinimumSize() {
  	    return getPreferredSize();
	}

	public void setVisible(int n) {
	    visibleRow = n;
	    repaint();
	}

	Dimension getEfectiveFormat() {
	    int r = rows, c = cols;
	    int items = data.size();

	    if (0 == rows) {
		// Divide rounding up
		r = (items - 1) / cols + 1;
	    }
	    if (0 == cols) {
		// Divide rounding up
		c = (items - 1) / rows + 1;
	    }

	    return new Dimension(c, r);
	}

	Point getCellFromCoord(Point coord) {
	    doCellSize();

	    Dimension d = getEfectiveFormat();
	    int r = d.height, c = d.width;

	    int w = 0;
	    int y = visibleRow + (int) coord.y / (cellHeight + 2 * ipady);

	    if (y >= r) {
		return new Point(-1, -1);
	    }

	    for (int x = 0; x < c; x++) {
		w += ((Integer) cellWidth.elementAt(x)).intValue() + 2 * ipadx;
		if (coord.x < w) {
		    return new Point(x, y);
		}
	    }

	    return new Point(-1, -1);
	}
    }

    class Cell {
	final String s;
	boolean selected;

	Cell(String s) {
	    this.s = s;
	    selected = false;
	}

	public void select(boolean selected) {
	    this.selected = selected;

	    int item_selected = selected ? ItemEvent.SELECTED : ItemEvent.DESELECTED;

	    processItemEvent(new ItemEvent(Table.this,
					   ItemEvent.ITEM_STATE_CHANGED,
					   this,
					   item_selected));
	}

	public boolean isSelected() {
	    return (selected);
	}

	public String toString() {
	    return s;
	}
    }
}
